"""
ToolUniverse Tools

Type-safe Python interface to 734 scientific tools.
Each tool is in its own module for minimal import overhead.

Usage:
    from tooluniverse.tools import ArXiv_search_papers
    result = ArXiv_search_papers(query="machine learning")
"""

# Import exceptions from main package

# Import shared client utilities
from ._shared_client import get_shared_client, reset_shared_client

# Import all tools
from .ADMETAI_predict_BBB_penetrance import ADMETAI_predict_BBB_penetrance
from .ADMETAI_predict_CYP_interactions import ADMETAI_predict_CYP_interactions
from .ADMETAI_predict_bioavailability import ADMETAI_predict_bioavailability
from .ADMETAI_predict_clearance_distribution import (
    ADMETAI_predict_clearance_distribution,
)
from .ADMETAI_predict_nuclear_receptor_activity import (
    ADMETAI_predict_nuclear_receptor_activity,
)
from .ADMETAI_predict_physicochemical_properties import (
    ADMETAI_predict_physicochemical_properties,
)
from .ADMETAI_predict_solubility_lipophilicity_hydration import (
    ADMETAI_predict_solubility_lipophilicity_hydration,
)
from .ADMETAI_predict_stress_response import ADMETAI_predict_stress_response
from .ADMETAI_predict_toxicity import ADMETAI_predict_toxicity
from .ADMETAnalyzerAgent import ADMETAnalyzerAgent
from .AdvancedCodeQualityAnalyzer import AdvancedCodeQualityAnalyzer
from .AdverseEventICDMapper import AdverseEventICDMapper
from .AdverseEventPredictionQuestionGenerator import (
    AdverseEventPredictionQuestionGenerator,
)
from .AdverseEventPredictionQuestionGeneratorWithContext import (
    AdverseEventPredictionQuestionGeneratorWithContext,
)
from .ArXiv_search_papers import ArXiv_search_papers
from .ArgumentDescriptionOptimizer import ArgumentDescriptionOptimizer
from .BLAST_nucleotide_search import BLAST_nucleotide_search
from .BLAST_protein_search import BLAST_protein_search
from .BioRxiv_search_preprints import BioRxiv_search_preprints
from .BiomarkerDiscoveryWorkflow import BiomarkerDiscoveryWorkflow
from .CMA_Guidelines_Search import CMA_Guidelines_Search
from .CORE_search_papers import CORE_search_papers
from .CallAgent import CallAgent
from .ChEMBL_search_similar_molecules import ChEMBL_search_similar_molecules
from .ClinicalTrialDesignAgent import ClinicalTrialDesignAgent
from .CodeQualityAnalyzer import CodeQualityAnalyzer
from .CompoundDiscoveryAgent import CompoundDiscoveryAgent
from .ComprehensiveDrugDiscoveryPipeline import ComprehensiveDrugDiscoveryPipeline
from .Crossref_search_works import Crossref_search_works
from .DBLP_search_publications import DBLP_search_publications
from .DOAJ_search_articles import DOAJ_search_articles
from .DailyMed_get_spl_by_setid import DailyMed_get_spl_by_setid
from .DailyMed_search_spls import DailyMed_search_spls
from .DataAnalysisValidityReviewer import DataAnalysisValidityReviewer
from .DescriptionAnalyzer import DescriptionAnalyzer
from .DescriptionQualityEvaluator import DescriptionQualityEvaluator
from .DiseaseAnalyzerAgent import DiseaseAnalyzerAgent
from .DomainExpertValidator import DomainExpertValidator
from .DrugInteractionAnalyzerAgent import DrugInteractionAnalyzerAgent
from .DrugOptimizationAgent import DrugOptimizationAgent
from .DrugSafetyAnalyzer import DrugSafetyAnalyzer
from .EMDB_get_structure import EMDB_get_structure
from .ENCODE_list_files import ENCODE_list_files
from .ENCODE_search_experiments import ENCODE_search_experiments
from .EthicalComplianceReviewer import EthicalComplianceReviewer
from .EuropePMC_Guidelines_Search import EuropePMC_Guidelines_Search
from .EuropePMC_search_articles import EuropePMC_search_articles
from .ExperimentalDesignScorer import ExperimentalDesignScorer
from .FAERS_count_additive_administration_routes import (
    FAERS_count_additive_administration_routes,
)
from .FAERS_count_additive_adverse_reactions import (
    FAERS_count_additive_adverse_reactions,
)
from .FAERS_count_additive_event_reports_by_country import (
    FAERS_count_additive_event_reports_by_country,
)
from .FAERS_count_additive_reaction_outcomes import (
    FAERS_count_additive_reaction_outcomes,
)
from .FAERS_count_additive_reports_by_reporter_country import (
    FAERS_count_additive_reports_by_reporter_country,
)
from .FAERS_count_additive_seriousness_classification import (
    FAERS_count_additive_seriousness_classification,
)
from .FAERS_count_country_by_drug_event import FAERS_count_country_by_drug_event
from .FAERS_count_death_related_by_drug import FAERS_count_death_related_by_drug
from .FAERS_count_drug_routes_by_event import FAERS_count_drug_routes_by_event
from .FAERS_count_drugs_by_drug_event import FAERS_count_drugs_by_drug_event
from .FAERS_count_outcomes_by_drug_event import FAERS_count_outcomes_by_drug_event
from .FAERS_count_patient_age_distribution import FAERS_count_patient_age_distribution
from .FAERS_count_reactions_by_drug_event import FAERS_count_reactions_by_drug_event
from .FAERS_count_reportercountry_by_drug_event import (
    FAERS_count_reportercountry_by_drug_event,
)
from .FAERS_count_seriousness_by_drug_event import FAERS_count_seriousness_by_drug_event
from .FDA_get_abuse_dependence_info_by_drug_name import (
    FDA_get_abuse_dependence_info_by_drug_name,
)
from .FDA_get_abuse_info_by_drug_name import FDA_get_abuse_info_by_drug_name
from .FDA_get_accessories_info_by_drug_name import FDA_get_accessories_info_by_drug_name
from .FDA_get_active_ingredient_info_by_drug_name import (
    FDA_get_active_ingredient_info_by_drug_name,
)
from .FDA_get_adverse_reactions_by_drug_name import (
    FDA_get_adverse_reactions_by_drug_name,
)
from .FDA_get_alarms_by_drug_name import FDA_get_alarms_by_drug_name
from .FDA_get_animal_pharmacology_info_by_drug_name import (
    FDA_get_animal_pharmacology_info_by_drug_name,
)
from .FDA_get_assembly_installation_info_by_drug_name import (
    FDA_get_assembly_installation_info_by_drug_name,
)
from .FDA_get_boxed_warning_info_by_drug_name import (
    FDA_get_boxed_warning_info_by_drug_name,
)
from .FDA_get_brand_name_generic_name import FDA_get_brand_name_generic_name
from .FDA_get_calibration_instructions_by_drug_name import (
    FDA_get_calibration_instructions_by_drug_name,
)
from .FDA_get_carcinogenic_mutagenic_fertility_by_drug_name import (
    FDA_get_carcinogenic_mutagenic_fertility_by_drug_name,
)
from .FDA_get_child_safety_info_by_drug_name import (
    FDA_get_child_safety_info_by_drug_name,
)
from .FDA_get_clinical_pharmacology_by_drug_name import (
    FDA_get_clinical_pharmacology_by_drug_name,
)
from .FDA_get_clinical_studies_info_by_drug_name import (
    FDA_get_clinical_studies_info_by_drug_name,
)
from .FDA_get_contact_for_questions_info_by_drug_name import (
    FDA_get_contact_for_questions_info_by_drug_name,
)
from .FDA_get_contraindications_by_drug_name import (
    FDA_get_contraindications_by_drug_name,
)
from .FDA_get_controlled_substance_DEA_schedule_info_by_drug_name import (
    FDA_get_controlled_substance_DEA_schedule_info_by_drug_name,
)
from .FDA_get_dear_health_care_provider_letter_info_by_drug_name import (
    FDA_get_dear_health_care_provider_letter_info_by_drug_name,
)
from .FDA_get_dependence_info_by_drug_name import FDA_get_dependence_info_by_drug_name
from .FDA_get_disposal_info_by_drug_name import FDA_get_disposal_info_by_drug_name
from .FDA_get_do_not_use_info_by_drug_name import FDA_get_do_not_use_info_by_drug_name
from .FDA_get_document_id_by_drug_name import FDA_get_document_id_by_drug_name
from .FDA_get_dosage_and_storage_information_by_drug_name import (
    FDA_get_dosage_and_storage_information_by_drug_name,
)
from .FDA_get_dosage_forms_and_strengths_by_drug_name import (
    FDA_get_dosage_forms_and_strengths_by_drug_name,
)
from .FDA_get_drug_generic_name import FDA_get_drug_generic_name
from .FDA_get_drug_interactions_by_drug_name import (
    FDA_get_drug_interactions_by_drug_name,
)
from .FDA_get_drug_name_by_SPL_ID import FDA_get_drug_name_by_SPL_ID
from .FDA_get_drug_name_by_adverse_reaction import FDA_get_drug_name_by_adverse_reaction
from .FDA_get_drug_name_by_calibration_instructions import (
    FDA_get_drug_name_by_calibration_instructions,
)
from .FDA_get_drug_name_by_dependence_info import FDA_get_drug_name_by_dependence_info
from .FDA_get_drug_name_by_document_id import FDA_get_drug_name_by_document_id
from .FDA_get_drug_name_by_dosage_info import FDA_get_drug_name_by_dosage_info
from .FDA_get_drug_name_by_environmental_warning import (
    FDA_get_drug_name_by_environmental_warning,
)
from .FDA_get_drug_name_by_inactive_ingredient import (
    FDA_get_drug_name_by_inactive_ingredient,
)
from .FDA_get_drug_name_by_info_on_conditions_for_doctor_consultation import (
    FDA_get_drug_name_by_info_on_conditions_for_doctor_consultation,
)
from .FDA_get_drug_name_by_labor_and_delivery_info import (
    FDA_get_drug_name_by_labor_and_delivery_info,
)
from .FDA_get_drug_name_by_microbiology import FDA_get_drug_name_by_microbiology
from .FDA_get_drug_name_by_other_safety_info import (
    FDA_get_drug_name_by_other_safety_info,
)
from .FDA_get_drug_name_by_pharmacodynamics import FDA_get_drug_name_by_pharmacodynamics
from .FDA_get_drug_name_by_pharmacogenomics import FDA_get_drug_name_by_pharmacogenomics
from .FDA_get_drug_name_by_precautions import FDA_get_drug_name_by_precautions
from .FDA_get_drug_name_by_pregnancy_or_breastfeeding_info import (
    FDA_get_drug_name_by_pregnancy_or_breastfeeding_info,
)
from .FDA_get_drug_name_by_principal_display_panel import (
    FDA_get_drug_name_by_principal_display_panel,
)
from .FDA_get_drug_name_by_reference import FDA_get_drug_name_by_reference
from .FDA_get_drug_name_by_set_id import FDA_get_drug_name_by_set_id
from .FDA_get_drug_name_by_stop_use_info import FDA_get_drug_name_by_stop_use_info
from .FDA_get_drug_name_by_storage_and_handling_info import (
    FDA_get_drug_name_by_storage_and_handling_info,
)
from .FDA_get_drug_name_by_warnings import FDA_get_drug_name_by_warnings
from .FDA_get_drug_name_from_patient_package_insert import (
    FDA_get_drug_name_from_patient_package_insert,
)
from .FDA_get_drug_names_by_abuse_dependence_info import (
    FDA_get_drug_names_by_abuse_dependence_info,
)
from .FDA_get_drug_names_by_abuse_info import FDA_get_drug_names_by_abuse_info
from .FDA_get_drug_names_by_accessories import FDA_get_drug_names_by_accessories
from .FDA_get_drug_names_by_active_ingredient import (
    FDA_get_drug_names_by_active_ingredient,
)
from .FDA_get_drug_names_by_alarm import FDA_get_drug_names_by_alarm
from .FDA_get_drug_names_by_animal_pharmacology_info import (
    FDA_get_drug_names_by_animal_pharmacology_info,
)
from .FDA_get_drug_names_by_application_number_NDC_number import (
    FDA_get_drug_names_by_application_number_NDC_number,
)
from .FDA_get_drug_names_by_assembly_installation_info import (
    FDA_get_drug_names_by_assembly_installation_info,
)
from .FDA_get_drug_names_by_boxed_warning import FDA_get_drug_names_by_boxed_warning
from .FDA_get_drug_names_by_child_safety_info import (
    FDA_get_drug_names_by_child_safety_info,
)
from .FDA_get_drug_names_by_clinical_pharmacology import (
    FDA_get_drug_names_by_clinical_pharmacology,
)
from .FDA_get_drug_names_by_clinical_studies import (
    FDA_get_drug_names_by_clinical_studies,
)
from .FDA_get_drug_names_by_consulting_doctor_pharmacist_info import (
    FDA_get_drug_names_by_consulting_doctor_pharmacist_info,
)
from .FDA_get_drug_names_by_contraindications import (
    FDA_get_drug_names_by_contraindications,
)
from .FDA_get_drug_names_by_controlled_substance_DEA_schedule import (
    FDA_get_drug_names_by_controlled_substance_DEA_schedule,
)
from .FDA_get_drug_names_by_dear_health_care_provider_letter_info import (
    FDA_get_drug_names_by_dear_health_care_provider_letter_info,
)
from .FDA_get_drug_names_by_disposal_info import FDA_get_drug_names_by_disposal_info
from .FDA_get_drug_names_by_dosage_forms_and_strengths_info import (
    FDA_get_drug_names_by_dosage_forms_and_strengths_info,
)
from .FDA_get_drug_names_by_drug_interactions import (
    FDA_get_drug_names_by_drug_interactions,
)
from .FDA_get_drug_names_by_effective_time import FDA_get_drug_names_by_effective_time
from .FDA_get_drug_names_by_food_safety_warnings import (
    FDA_get_drug_names_by_food_safety_warnings,
)
from .FDA_get_drug_names_by_general_precautions import (
    FDA_get_drug_names_by_general_precautions,
)
from .FDA_get_drug_names_by_geriatric_use import FDA_get_drug_names_by_geriatric_use
from .FDA_get_drug_names_by_health_claim import FDA_get_drug_names_by_health_claim
from .FDA_get_drug_names_by_indication import FDA_get_drug_names_by_indication
from .FDA_get_drug_names_by_info_for_nursing_mothers import (
    FDA_get_drug_names_by_info_for_nursing_mothers,
)
from .FDA_get_drug_names_by_information_for_owners_or_caregivers import (
    FDA_get_drug_names_by_information_for_owners_or_caregivers,
)
from .FDA_get_drug_names_by_ingredient import FDA_get_drug_names_by_ingredient
from .FDA_get_drug_names_by_instructions_for_use import (
    FDA_get_drug_names_by_instructions_for_use,
)
from .FDA_get_drug_names_by_lab_test_interference import (
    FDA_get_drug_names_by_lab_test_interference,
)
from .FDA_get_drug_names_by_lab_tests import FDA_get_drug_names_by_lab_tests
from .FDA_get_drug_names_by_mechanism_of_action import (
    FDA_get_drug_names_by_mechanism_of_action,
)
from .FDA_get_drug_names_by_medication_guide import (
    FDA_get_drug_names_by_medication_guide,
)
from .FDA_get_drug_names_by_nonclinical_toxicology_info import (
    FDA_get_drug_names_by_nonclinical_toxicology_info,
)
from .FDA_get_drug_names_by_nonteratogenic_effects import (
    FDA_get_drug_names_by_nonteratogenic_effects,
)
from .FDA_get_drug_names_by_overdosage_info import FDA_get_drug_names_by_overdosage_info
from .FDA_get_drug_names_by_pediatric_use import FDA_get_drug_names_by_pediatric_use
from .FDA_get_drug_names_by_pharmacokinetics import (
    FDA_get_drug_names_by_pharmacokinetics,
)
from .FDA_get_drug_names_by_population_use import FDA_get_drug_names_by_population_use
from .FDA_get_drug_names_by_pregnancy_effects_info import (
    FDA_get_drug_names_by_pregnancy_effects_info,
)
from .FDA_get_drug_names_by_residue_warning import FDA_get_drug_names_by_residue_warning
from .FDA_get_drug_names_by_risk import FDA_get_drug_names_by_risk
from .FDA_get_drug_names_by_route import FDA_get_drug_names_by_route
from .FDA_get_drug_names_by_safe_handling_warning import (
    FDA_get_drug_names_by_safe_handling_warning,
)
from .FDA_get_drug_names_by_safety_summary import FDA_get_drug_names_by_safety_summary
from .FDA_get_drug_names_by_spl_indexing_data_elements import (
    FDA_get_drug_names_by_spl_indexing_data_elements,
)
from .FDA_get_drug_names_by_teratogenic_effects import (
    FDA_get_drug_names_by_teratogenic_effects,
)
from .FDA_get_drug_names_by_user_safety_warning import (
    FDA_get_drug_names_by_user_safety_warning,
)
from .FDA_get_drug_names_by_warnings_and_cautions import (
    FDA_get_drug_names_by_warnings_and_cautions,
)
from .FDA_get_drugs_by_carcinogenic_mutagenic_fertility import (
    FDA_get_drugs_by_carcinogenic_mutagenic_fertility,
)
from .FDA_get_effective_time_by_drug_name import FDA_get_effective_time_by_drug_name
from .FDA_get_environmental_warning_by_drug_name import (
    FDA_get_environmental_warning_by_drug_name,
)
from .FDA_get_general_precautions_by_drug_name import (
    FDA_get_general_precautions_by_drug_name,
)
from .FDA_get_geriatric_use_info_by_drug_name import (
    FDA_get_geriatric_use_info_by_drug_name,
)
from .FDA_get_health_claims_by_drug_name import FDA_get_health_claims_by_drug_name
from .FDA_get_inactive_ingredient_info_by_drug_name import (
    FDA_get_inactive_ingredient_info_by_drug_name,
)
from .FDA_get_indications_by_drug_name import FDA_get_indications_by_drug_name
from .FDA_get_info_for_nursing_mothers_by_drug_name import (
    FDA_get_info_for_nursing_mothers_by_drug_name,
)
from .FDA_get_info_for_patients_by_drug_name import (
    FDA_get_info_for_patients_by_drug_name,
)
from .FDA_get_info_on_conditions_for_doctor_consultation_by_drug_name import (
    FDA_get_info_on_conditions_for_doctor_consultation_by_drug_name,
)
from .FDA_get_info_on_consulting_doctor_pharmacist_by_drug_name import (
    FDA_get_info_on_consulting_doctor_pharmacist_by_drug_name,
)
from .FDA_get_information_for_owners_or_caregivers_by_drug_name import (
    FDA_get_information_for_owners_or_caregivers_by_drug_name,
)
from .FDA_get_ingredients_by_drug_name import FDA_get_ingredients_by_drug_name
from .FDA_get_instructions_for_use_by_drug_name import (
    FDA_get_instructions_for_use_by_drug_name,
)
from .FDA_get_lab_test_interference_info_by_drug_name import (
    FDA_get_lab_test_interference_info_by_drug_name,
)
from .FDA_get_lab_tests_by_drug_name import FDA_get_lab_tests_by_drug_name
from .FDA_get_labor_and_delivery_info_by_drug_name import (
    FDA_get_labor_and_delivery_info_by_drug_name,
)
from .FDA_get_manufacturer_name_NDC_number_by_drug_name import (
    FDA_get_manufacturer_name_NDC_number_by_drug_name,
)
from .FDA_get_mechanism_of_action_by_drug_name import (
    FDA_get_mechanism_of_action_by_drug_name,
)
from .FDA_get_medication_guide_info_by_drug_name import (
    FDA_get_medication_guide_info_by_drug_name,
)
from .FDA_get_microbiology_info_by_drug_name import (
    FDA_get_microbiology_info_by_drug_name,
)
from .FDA_get_nonclinical_toxicology_info_by_drug_name import (
    FDA_get_nonclinical_toxicology_info_by_drug_name,
)
from .FDA_get_nonteratogenic_effects_by_drug_name import (
    FDA_get_nonteratogenic_effects_by_drug_name,
)
from .FDA_get_other_safety_info_by_drug_name import (
    FDA_get_other_safety_info_by_drug_name,
)
from .FDA_get_overdosage_info_by_drug_name import FDA_get_overdosage_info_by_drug_name
from .FDA_get_patient_package_insert_from_drug_name import (
    FDA_get_patient_package_insert_from_drug_name,
)
from .FDA_get_pediatric_use_info_by_drug_name import (
    FDA_get_pediatric_use_info_by_drug_name,
)
from .FDA_get_pharmacodynamics_by_drug_name import FDA_get_pharmacodynamics_by_drug_name
from .FDA_get_pharmacogenomics_info_by_drug_name import (
    FDA_get_pharmacogenomics_info_by_drug_name,
)
from .FDA_get_pharmacokinetics_by_drug_name import FDA_get_pharmacokinetics_by_drug_name
from .FDA_get_population_use_info_by_drug_name import (
    FDA_get_population_use_info_by_drug_name,
)
from .FDA_get_precautions_by_drug_name import FDA_get_precautions_by_drug_name
from .FDA_get_pregnancy_effects_info_by_drug_name import (
    FDA_get_pregnancy_effects_info_by_drug_name,
)
from .FDA_get_pregnancy_or_breastfeeding_info_by_drug_name import (
    FDA_get_pregnancy_or_breastfeeding_info_by_drug_name,
)
from .FDA_get_principal_display_panel_by_drug_name import (
    FDA_get_principal_display_panel_by_drug_name,
)
from .FDA_get_purpose_info_by_drug_name import FDA_get_purpose_info_by_drug_name
from .FDA_get_recent_changes_by_drug_name import FDA_get_recent_changes_by_drug_name
from .FDA_get_reference_info_by_drug_name import FDA_get_reference_info_by_drug_name
from .FDA_get_residue_warning_by_drug_name import FDA_get_residue_warning_by_drug_name
from .FDA_get_risk_info_by_drug_name import FDA_get_risk_info_by_drug_name
from .FDA_get_route_info_by_drug_name import FDA_get_route_info_by_drug_name
from .FDA_get_safe_handling_warnings_by_drug_name import (
    FDA_get_safe_handling_warnings_by_drug_name,
)
from .FDA_get_safety_summary_by_drug_name import FDA_get_safety_summary_by_drug_name
from .FDA_get_spl_indexing_data_elements_by_drug_name import (
    FDA_get_spl_indexing_data_elements_by_drug_name,
)
from .FDA_get_spl_unclassified_section_by_drug_name import (
    FDA_get_spl_unclassified_section_by_drug_name,
)
from .FDA_get_stop_use_info_by_drug_name import FDA_get_stop_use_info_by_drug_name
from .FDA_get_storage_and_handling_info_by_drug_name import (
    FDA_get_storage_and_handling_info_by_drug_name,
)
from .FDA_get_teratogenic_effects_by_drug_name import (
    FDA_get_teratogenic_effects_by_drug_name,
)
from .FDA_get_user_safety_warning_by_drug_names import (
    FDA_get_user_safety_warning_by_drug_names,
)
from .FDA_get_warnings_and_cautions_by_drug_name import (
    FDA_get_warnings_and_cautions_by_drug_name,
)
from .FDA_get_warnings_by_drug_name import FDA_get_warnings_by_drug_name
from .FDA_get_when_using_info import FDA_get_when_using_info
from .FDA_retrieve_device_use_by_drug_name import FDA_retrieve_device_use_by_drug_name
from .FDA_retrieve_drug_name_by_device_use import FDA_retrieve_drug_name_by_device_use
from .FDA_retrieve_drug_names_by_patient_medication_info import (
    FDA_retrieve_drug_names_by_patient_medication_info,
)
from .FDA_retrieve_patient_medication_info_by_drug_name import (
    FDA_retrieve_patient_medication_info_by_drug_name,
)
from .Fatcat_search_scholar import Fatcat_search_scholar
from .Finish import Finish
from .GBIF_search_occurrences import GBIF_search_occurrences
from .GBIF_search_species import GBIF_search_species
from .GDC_list_files import GDC_list_files
from .GDC_search_cases import GDC_search_cases
from .GIN_Guidelines_Search import GIN_Guidelines_Search
from .GO_get_annotations_for_gene import GO_get_annotations_for_gene
from .GO_get_genes_for_term import GO_get_genes_for_term
from .GO_get_term_by_id import GO_get_term_by_id
from .GO_get_term_details import GO_get_term_details
from .GO_search_terms import GO_search_terms
from .GTEx_get_expression_summary import GTEx_get_expression_summary
from .GTEx_query_eqtl import GTEx_query_eqtl
from .GWAS_search_associations_by_gene import GWAS_search_associations_by_gene
from .GtoPdb_get_targets import GtoPdb_get_targets
from .HAL_search_archive import HAL_search_archive
from .HPA_get_biological_processes_by_gene import HPA_get_biological_processes_by_gene
from .HPA_get_cancer_prognostics_by_gene import HPA_get_cancer_prognostics_by_gene
from .HPA_get_comparative_expression_by_gene_and_cellline import (
    HPA_get_comparative_expression_by_gene_and_cellline,
)
from .HPA_get_comprehensive_gene_details_by_ensembl_id import (
    HPA_get_comprehensive_gene_details_by_ensembl_id,
)
from .HPA_get_contextual_biological_process_analysis import (
    HPA_get_contextual_biological_process_analysis,
)
from .HPA_get_disease_expression_by_gene_tissue_disease import (
    HPA_get_disease_expression_by_gene_tissue_disease,
)
from .HPA_get_gene_basic_info_by_ensembl_id import HPA_get_gene_basic_info_by_ensembl_id
from .HPA_get_gene_tsv_data_by_ensembl_id import HPA_get_gene_tsv_data_by_ensembl_id
from .HPA_get_protein_interactions_by_gene import HPA_get_protein_interactions_by_gene
from .HPA_get_rna_expression_by_source import HPA_get_rna_expression_by_source
from .HPA_get_rna_expression_in_specific_tissues import (
    HPA_get_rna_expression_in_specific_tissues,
)
from .HPA_get_subcellular_location import HPA_get_subcellular_location
from .HPA_search_genes_by_query import HPA_search_genes_by_query
from .HypothesisGenerator import HypothesisGenerator
from .InterPro_get_domain_details import InterPro_get_domain_details
from .InterPro_get_protein_domains import InterPro_get_protein_domains
from .InterPro_search_domains import InterPro_search_domains
from .JASPAR_get_transcription_factors import JASPAR_get_transcription_factors
from .LabelGenerator import LabelGenerator
from .LiteratureContextReviewer import LiteratureContextReviewer
from .LiteratureSearchTool import LiteratureSearchTool
from .LiteratureSynthesisAgent import LiteratureSynthesisAgent
from .MGnify_list_analyses import MGnify_list_analyses
from .MGnify_search_studies import MGnify_search_studies
from .MPD_get_phenotype_data import MPD_get_phenotype_data
from .MedRxiv_search_preprints import MedRxiv_search_preprints
from .MedicalLiteratureReviewer import MedicalLiteratureReviewer
from .MedicalTermNormalizer import MedicalTermNormalizer
from .MedlinePlus_connect_lookup_by_code import MedlinePlus_connect_lookup_by_code
from .MedlinePlus_get_genetics_condition_by_name import (
    MedlinePlus_get_genetics_condition_by_name,
)
from .MedlinePlus_get_genetics_gene_by_name import MedlinePlus_get_genetics_gene_by_name
from .MedlinePlus_get_genetics_index import MedlinePlus_get_genetics_index
from .MedlinePlus_search_topics_by_keyword import MedlinePlus_search_topics_by_keyword
from .MethodologyRigorReviewer import MethodologyRigorReviewer
from .NICE_Clinical_Guidelines_Search import NICE_Clinical_Guidelines_Search
from .NICE_Guideline_Full_Text import NICE_Guideline_Full_Text
from .NoveltySignificanceReviewer import NoveltySignificanceReviewer
from .OBIS_search_occurrences import OBIS_search_occurrences
from .OBIS_search_taxa import OBIS_search_taxa
from .OSF_search_preprints import OSF_search_preprints
from .OSL_get_efo_id_by_disease_name import OSL_get_efo_id_by_disease_name
from .OpenAIRE_search_publications import OpenAIRE_search_publications
from .OpenAlex_Guidelines_Search import OpenAlex_Guidelines_Search
from .OpenTargets_drug_pharmacogenomics_data import (
    OpenTargets_drug_pharmacogenomics_data,
)
from .OpenTargets_get_approved_indications_by_drug_chemblId import (
    OpenTargets_get_approved_indications_by_drug_chemblId,
)
from .OpenTargets_get_associated_diseases_by_drug_chemblId import (
    OpenTargets_get_associated_diseases_by_drug_chemblId,
)
from .OpenTargets_get_associated_drugs_by_disease_efoId import (
    OpenTargets_get_associated_drugs_by_disease_efoId,
)
from .OpenTargets_get_associated_drugs_by_target_ensemblID import (
    OpenTargets_get_associated_drugs_by_target_ensemblID,
)
from .OpenTargets_get_associated_phenotypes_by_disease_efoId import (
    OpenTargets_get_associated_phenotypes_by_disease_efoId,
)
from .OpenTargets_get_associated_targets_by_disease_efoId import (
    OpenTargets_get_associated_targets_by_disease_efoId,
)
from .OpenTargets_get_associated_targets_by_drug_chemblId import (
    OpenTargets_get_associated_targets_by_drug_chemblId,
)
from .OpenTargets_get_biological_mouse_models_by_ensemblID import (
    OpenTargets_get_biological_mouse_models_by_ensemblID,
)
from .OpenTargets_get_chemical_probes_by_target_ensemblID import (
    OpenTargets_get_chemical_probes_by_target_ensemblID,
)
from .OpenTargets_get_disease_ancestors_parents_by_efoId import (
    OpenTargets_get_disease_ancestors_parents_by_efoId,
)
from .OpenTargets_get_disease_descendants_children_by_efoId import (
    OpenTargets_get_disease_descendants_children_by_efoId,
)
from .OpenTargets_get_disease_description_by_efoId import (
    OpenTargets_get_disease_description_by_efoId,
)
from .OpenTargets_get_disease_id_description_by_name import (
    OpenTargets_get_disease_id_description_by_name,
)
from .OpenTargets_get_disease_ids_by_efoId import OpenTargets_get_disease_ids_by_efoId
from .OpenTargets_get_disease_ids_by_name import OpenTargets_get_disease_ids_by_name
from .OpenTargets_get_disease_locations_by_efoId import (
    OpenTargets_get_disease_locations_by_efoId,
)
from .OpenTargets_get_disease_synonyms_by_efoId import (
    OpenTargets_get_disease_synonyms_by_efoId,
)
from .OpenTargets_get_disease_therapeutic_areas_by_efoId import (
    OpenTargets_get_disease_therapeutic_areas_by_efoId,
)
from .OpenTargets_get_diseases_phenotypes_by_target_ensembl import (
    OpenTargets_get_diseases_phenotypes_by_target_ensembl,
)
from .OpenTargets_get_drug_adverse_events_by_chemblId import (
    OpenTargets_get_drug_adverse_events_by_chemblId,
)
from .OpenTargets_get_drug_approval_status_by_chemblId import (
    OpenTargets_get_drug_approval_status_by_chemblId,
)
from .OpenTargets_get_drug_chembId_by_generic_name import (
    OpenTargets_get_drug_chembId_by_generic_name,
)
from .OpenTargets_get_drug_description_by_chemblId import (
    OpenTargets_get_drug_description_by_chemblId,
)
from .OpenTargets_get_drug_id_description_by_name import (
    OpenTargets_get_drug_id_description_by_name,
)
from .OpenTargets_get_drug_indications_by_chemblId import (
    OpenTargets_get_drug_indications_by_chemblId,
)
from .OpenTargets_get_drug_mechanisms_of_action_by_chemblId import (
    OpenTargets_get_drug_mechanisms_of_action_by_chemblId,
)
from .OpenTargets_get_drug_synonyms_by_chemblId import (
    OpenTargets_get_drug_synonyms_by_chemblId,
)
from .OpenTargets_get_drug_trade_names_by_chemblId import (
    OpenTargets_get_drug_trade_names_by_chemblId,
)
from .OpenTargets_get_drug_warnings_by_chemblId import (
    OpenTargets_get_drug_warnings_by_chemblId,
)
from .OpenTargets_get_drug_withdrawn_blackbox_status_by_chemblId import (
    OpenTargets_get_drug_withdrawn_blackbox_status_by_chemblId,
)
from .OpenTargets_get_gene_ontology_terms_by_goID import (
    OpenTargets_get_gene_ontology_terms_by_goID,
)
from .OpenTargets_get_known_drugs_by_drug_chemblId import (
    OpenTargets_get_known_drugs_by_drug_chemblId,
)
from .OpenTargets_get_parent_child_molecules_by_drug_chembl_ID import (
    OpenTargets_get_parent_child_molecules_by_drug_chembl_ID,
)
from .OpenTargets_get_publications_by_disease_efoId import (
    OpenTargets_get_publications_by_disease_efoId,
)
from .OpenTargets_get_publications_by_drug_chemblId import (
    OpenTargets_get_publications_by_drug_chemblId,
)
from .OpenTargets_get_publications_by_target_ensemblID import (
    OpenTargets_get_publications_by_target_ensemblID,
)
from .OpenTargets_get_similar_entities_by_disease_efoId import (
    OpenTargets_get_similar_entities_by_disease_efoId,
)
from .OpenTargets_get_similar_entities_by_drug_chemblId import (
    OpenTargets_get_similar_entities_by_drug_chemblId,
)
from .OpenTargets_get_similar_entities_by_target_ensemblID import (
    OpenTargets_get_similar_entities_by_target_ensemblID,
)
from .OpenTargets_get_target_classes_by_ensemblID import (
    OpenTargets_get_target_classes_by_ensemblID,
)
from .OpenTargets_get_target_constraint_info_by_ensemblID import (
    OpenTargets_get_target_constraint_info_by_ensemblID,
)
from .OpenTargets_get_target_enabling_packages_by_ensemblID import (
    OpenTargets_get_target_enabling_packages_by_ensemblID,
)
from .OpenTargets_get_target_gene_ontology_by_ensemblID import (
    OpenTargets_get_target_gene_ontology_by_ensemblID,
)
from .OpenTargets_get_target_genomic_location_by_ensemblID import (
    OpenTargets_get_target_genomic_location_by_ensemblID,
)
from .OpenTargets_get_target_homologues_by_ensemblID import (
    OpenTargets_get_target_homologues_by_ensemblID,
)
from .OpenTargets_get_target_id_description_by_name import (
    OpenTargets_get_target_id_description_by_name,
)
from .OpenTargets_get_target_interactions_by_ensemblID import (
    OpenTargets_get_target_interactions_by_ensemblID,
)
from .OpenTargets_get_target_safety_profile_by_ensemblID import (
    OpenTargets_get_target_safety_profile_by_ensemblID,
)
from .OpenTargets_get_target_subcellular_locations_by_ensemblID import (
    OpenTargets_get_target_subcellular_locations_by_ensemblID,
)
from .OpenTargets_get_target_synonyms_by_ensemblID import (
    OpenTargets_get_target_synonyms_by_ensemblID,
)
from .OpenTargets_get_target_tractability_by_ensemblID import (
    OpenTargets_get_target_tractability_by_ensemblID,
)
from .OpenTargets_map_any_disease_id_to_all_other_ids import (
    OpenTargets_map_any_disease_id_to_all_other_ids,
)
from .OpenTargets_multi_entity_search_by_query_string import (
    OpenTargets_multi_entity_search_by_query_string,
)
from .OpenTargets_search_category_counts_by_query_string import (
    OpenTargets_search_category_counts_by_query_string,
)
from .OpenTargets_target_disease_evidence import OpenTargets_target_disease_evidence
from .OutputSummarizationComposer import OutputSummarizationComposer
from .PMC_search_papers import PMC_search_papers
from .PRIDE_search_proteomics import PRIDE_search_proteomics
from .PackageAnalyzer import PackageAnalyzer
from .Paleobiology_get_fossils import Paleobiology_get_fossils
from .ProtocolOptimizer import ProtocolOptimizer
from .PubChem_get_CID_by_SMILES import PubChem_get_CID_by_SMILES
from .PubChem_get_CID_by_compound_name import PubChem_get_CID_by_compound_name
from .PubChem_get_associated_patents_by_CID import PubChem_get_associated_patents_by_CID
from .PubChem_get_compound_2D_image_by_CID import PubChem_get_compound_2D_image_by_CID
from .PubChem_get_compound_properties_by_CID import (
    PubChem_get_compound_properties_by_CID,
)
from .PubChem_get_compound_synonyms_by_CID import PubChem_get_compound_synonyms_by_CID
from .PubChem_get_compound_xrefs_by_CID import PubChem_get_compound_xrefs_by_CID
from .PubChem_search_compounds_by_similarity import (
    PubChem_search_compounds_by_similarity,
)
from .PubChem_search_compounds_by_substructure import (
    PubChem_search_compounds_by_substructure,
)
from .PubMed_Guidelines_Search import PubMed_Guidelines_Search
from .PubMed_search_articles import PubMed_search_articles
from .PubTator3_EntityAutocomplete import PubTator3_EntityAutocomplete
from .PubTator3_LiteratureSearch import PubTator3_LiteratureSearch
from .PyPIPackageInspector import PyPIPackageInspector
from .QuestionRephraser import QuestionRephraser
from .RNAcentral_get_by_accession import RNAcentral_get_by_accession
from .RNAcentral_search import RNAcentral_search
from .ReMap_get_transcription_factor_binding import (
    ReMap_get_transcription_factor_binding,
)
from .Reactome_get_pathway_reactions import Reactome_get_pathway_reactions
from .ReferenceInfoAnalyzer import ReferenceInfoAnalyzer
from .RegulomeDB_query_variant import RegulomeDB_query_variant
from .ReproducibilityTransparencyReviewer import ReproducibilityTransparencyReviewer
from .ResultsInterpretationReviewer import ResultsInterpretationReviewer
from .SCREEN_get_regulatory_elements import SCREEN_get_regulatory_elements
from .ScientificTextSummarizer import ScientificTextSummarizer
from .SemanticScholar_search_papers import SemanticScholar_search_papers
from .TRIP_Database_Guidelines_Search import TRIP_Database_Guidelines_Search
from .TestCaseGenerator import TestCaseGenerator
from .TestResultsAnalyzer import TestResultsAnalyzer
from .ToolCompatibilityAnalyzer import ToolCompatibilityAnalyzer
from .ToolDescriptionOptimizer import ToolDescriptionOptimizer
from .ToolDiscover import ToolDiscover
from .ToolGraphComposer import ToolGraphComposer
from .ToolGraphGenerationPipeline import ToolGraphGenerationPipeline
from .ToolMetadataGenerationPipeline import ToolMetadataGenerationPipeline
from .ToolMetadataGenerator import ToolMetadataGenerator
from .ToolMetadataStandardizer import ToolMetadataStandardizer
from .ToolOutputSummarizer import ToolOutputSummarizer
from .ToolQualityEvaluator import ToolQualityEvaluator
from .ToolRelationshipDetector import ToolRelationshipDetector
from .Tool_Finder import Tool_Finder
from .Tool_Finder_Keyword import Tool_Finder_Keyword
from .Tool_Finder_LLM import Tool_Finder_LLM
from .Tool_RAG import Tool_RAG
from .UniProt_get_alternative_names_by_accession import (
    UniProt_get_alternative_names_by_accession,
)
from .UniProt_get_disease_variants_by_accession import (
    UniProt_get_disease_variants_by_accession,
)
from .UniProt_get_entry_by_accession import UniProt_get_entry_by_accession
from .UniProt_get_function_by_accession import UniProt_get_function_by_accession
from .UniProt_get_isoform_ids_by_accession import UniProt_get_isoform_ids_by_accession
from .UniProt_get_organism_by_accession import UniProt_get_organism_by_accession
from .UniProt_get_ptm_processing_by_accession import (
    UniProt_get_ptm_processing_by_accession,
)
from .UniProt_get_recommended_name_by_accession import (
    UniProt_get_recommended_name_by_accession,
)
from .UniProt_get_sequence_by_accession import UniProt_get_sequence_by_accession
from .UniProt_get_subcellular_location_by_accession import (
    UniProt_get_subcellular_location_by_accession,
)
from .UniProt_id_mapping import UniProt_id_mapping
from .UniProt_search import UniProt_search
from .UnifiedToolGenerator import UnifiedToolGenerator
from .Unpaywall_check_oa_status import Unpaywall_check_oa_status
from .WHO_Guideline_Full_Text import WHO_Guideline_Full_Text
from .WHO_Guidelines_Search import WHO_Guidelines_Search
from .WikiPathways_get_pathway import WikiPathways_get_pathway
from .WikiPathways_search import WikiPathways_search
from .Wikidata_SPARQL_query import Wikidata_SPARQL_query
from .WoRMS_search_species import WoRMS_search_species
from .WritingPresentationReviewer import WritingPresentationReviewer
from .XMLToolOptimizer import XMLToolOptimizer
from .Zenodo_search_records import Zenodo_search_records
from .advanced_literature_search_agent import advanced_literature_search_agent
from .alphafold_get_annotations import alphafold_get_annotations
from .alphafold_get_prediction import alphafold_get_prediction
from .alphafold_get_summary import alphafold_get_summary
from .cBioPortal_get_cancer_studies import cBioPortal_get_cancer_studies
from .cBioPortal_get_mutations import cBioPortal_get_mutations
from .call_agentic_human import call_agentic_human
from .cancer_biomarkers_disease_target_score import (
    cancer_biomarkers_disease_target_score,
)
from .cancer_gene_census_disease_target_score import (
    cancer_gene_census_disease_target_score,
)
from .cellosaurus_get_cell_line_info import cellosaurus_get_cell_line_info
from .cellosaurus_query_converter import cellosaurus_query_converter
from .cellosaurus_search_cell_lines import cellosaurus_search_cell_lines
from .chembl_disease_target_score import chembl_disease_target_score
from .clinvar_get_clinical_significance import clinvar_get_clinical_significance
from .clinvar_get_variant_details import clinvar_get_variant_details
from .clinvar_search_variants import clinvar_search_variants
from .convert_to_markdown import convert_to_markdown
from .dbsnp_get_frequencies import dbsnp_get_frequencies
from .dbsnp_get_variant_by_rsid import dbsnp_get_variant_by_rsid
from .dbsnp_search_by_gene import dbsnp_search_by_gene
from .dict_search import dict_search
from .dili_search import dili_search
from .diqt_search import diqt_search
from .disease_target_score import disease_target_score
from .download_binary_file import download_binary_file
from .download_file import download_file
from .download_text_content import download_text_content
from .drugbank_filter_drugs_by_name import drugbank_filter_drugs_by_name
from .drugbank_full_search import drugbank_full_search
from .drugbank_get_drug_basic_info_by_drug_name_or_drugbank_id import (
    drugbank_get_drug_basic_info_by_drug_name_or_drugbank_id,
)
from .drugbank_get_drug_chemistry_by_drug_name_or_drugbank_id import (
    drugbank_get_drug_chemistry_by_drug_name_or_drugbank_id,
)
from .drugbank_get_drug_desc_pharmacology_by_moa import (
    drugbank_get_drug_desc_pharmacology_by_moa,
)
from .drugbank_get_drug_interactions_by_drug_name_or_drugbank_id import (
    drugbank_get_drug_interactions_by_drug_name_or_drugbank_id,
)
from .drugbank_get_drug_name_and_description_by_indication import (
    drugbank_get_drug_name_and_description_by_indication,
)
from .drugbank_get_drug_name_and_description_by_pathway_name import (
    drugbank_get_drug_name_and_description_by_pathway_name,
)
from .drugbank_get_drug_name_and_description_by_target_name import (
    drugbank_get_drug_name_and_description_by_target_name,
)
from .drugbank_get_drug_products_by_name_or_drugbank_id import (
    drugbank_get_drug_products_by_name_or_drugbank_id,
)
from .drugbank_get_drug_references_by_drug_name_or_drugbank_id import (
    drugbank_get_drug_references_by_drug_name_or_drugbank_id,
)
from .drugbank_get_indications_by_drug_name_or_drugbank_id import (
    drugbank_get_indications_by_drug_name_or_drugbank_id,
)
from .drugbank_get_pathways_reactions_by_drug_or_id import (
    drugbank_get_pathways_reactions_by_drug_or_id,
)
from .drugbank_get_pharmacology_by_drug_name_or_drugbank_id import (
    drugbank_get_pharmacology_by_drug_name_or_drugbank_id,
)
from .drugbank_get_safety_by_drug_name_or_drugbank_id import (
    drugbank_get_safety_by_drug_name_or_drugbank_id,
)
from .drugbank_get_targets_by_drug_name_or_drugbank_id import (
    drugbank_get_targets_by_drug_name_or_drugbank_id,
)
from .drugbank_links_search import drugbank_links_search
from .drugbank_vocab_filter import drugbank_vocab_filter
from .drugbank_vocab_search import drugbank_vocab_search
from .dynamic_package_discovery import dynamic_package_discovery
from .embedding_database_add import embedding_database_add
from .embedding_database_create import embedding_database_create
from .embedding_database_load import embedding_database_load
from .embedding_database_search import embedding_database_search
from .embedding_sync_download import embedding_sync_download
from .embedding_sync_upload import embedding_sync_upload
from .enrichr_gene_enrichment_analysis import enrichr_gene_enrichment_analysis
from .ensembl_get_sequence import ensembl_get_sequence
from .ensembl_get_variants import ensembl_get_variants
from .ensembl_lookup_gene import ensembl_lookup_gene
from .europepmc_disease_target_score import europepmc_disease_target_score
from .eva_disease_target_score import eva_disease_target_score
from .eva_somatic_disease_target_score import eva_somatic_disease_target_score
from .expression_atlas_disease_target_score import expression_atlas_disease_target_score
from .extract_clinical_trial_adverse_events import extract_clinical_trial_adverse_events
from .extract_clinical_trial_outcomes import extract_clinical_trial_outcomes
from .genomics_england_disease_target_score import genomics_england_disease_target_score
from .geo_get_dataset_info import geo_get_dataset_info
from .geo_get_sample_info import geo_get_sample_info
from .geo_search_datasets import geo_search_datasets
from .get_HPO_ID_by_phenotype import get_HPO_ID_by_phenotype
from .get_albumentations_info import get_albumentations_info
from .get_altair_info import get_altair_info
from .get_anndata_info import get_anndata_info
from .get_arboreto_info import get_arboreto_info
from .get_arxiv_info import get_arxiv_info
from .get_ase_info import get_ase_info
from .get_assembly_info_by_pdb_id import get_assembly_info_by_pdb_id
from .get_assembly_summary import get_assembly_summary
from .get_astropy_info import get_astropy_info
from .get_binding_affinity_by_pdb_id import get_binding_affinity_by_pdb_id
from .get_biopandas_info import get_biopandas_info
from .get_biopython_info import get_biopython_info
from .get_bioservices_info import get_bioservices_info
from .get_biotite_info import get_biotite_info
from .get_bokeh_info import get_bokeh_info
from .get_brian2_info import get_brian2_info
from .get_cartopy_info import get_cartopy_info
from .get_catboost_info import get_catboost_info
from .get_cellpose_info import get_cellpose_info
from .get_cellrank_info import get_cellrank_info
from .get_cellxgene_census_info import get_cellxgene_census_info
from .get_cftime_info import get_cftime_info
from .get_chem_comp_audit_info import get_chem_comp_audit_info
from .get_chem_comp_charge_and_ambiguity import get_chem_comp_charge_and_ambiguity
from .get_chembl_webresource_client_info import get_chembl_webresource_client_info
from .get_citation_info_by_pdb_id import get_citation_info_by_pdb_id
from .get_clair3_info import get_clair3_info
from .get_clinical_trial_conditions_and_interventions import (
    get_clinical_trial_conditions_and_interventions,
)
from .get_clinical_trial_descriptions import get_clinical_trial_descriptions
from .get_clinical_trial_eligibility_criteria import (
    get_clinical_trial_eligibility_criteria,
)
from .get_clinical_trial_locations import get_clinical_trial_locations
from .get_clinical_trial_outcome_measures import get_clinical_trial_outcome_measures
from .get_clinical_trial_references import get_clinical_trial_references
from .get_clinical_trial_status_and_dates import get_clinical_trial_status_and_dates
from .get_cobra_info import get_cobra_info
from .get_cobrapy_info import get_cobrapy_info
from .get_cooler_info import get_cooler_info
from .get_core_refinement_statistics import get_core_refinement_statistics
from .get_cryosparc_tools_info import get_cryosparc_tools_info
from .get_crystal_growth_conditions_by_pdb_id import (
    get_crystal_growth_conditions_by_pdb_id,
)
from .get_crystallization_ph_by_pdb_id import get_crystallization_ph_by_pdb_id
from .get_crystallographic_properties_by_pdb_id import (
    get_crystallographic_properties_by_pdb_id,
)
from .get_cupy_info import get_cupy_info
from .get_cyvcf2_info import get_cyvcf2_info
from .get_dask_info import get_dask_info
from .get_datamol_info import get_datamol_info
from .get_datashader_info import get_datashader_info
from .get_deepchem_info import get_deepchem_info
from .get_deeppurpose_info import get_deeppurpose_info
from .get_deeptools_info import get_deeptools_info
from .get_deepxde_info import get_deepxde_info
from .get_dendropy_info import get_dendropy_info
from .get_descriptastorus_info import get_descriptastorus_info
from .get_diffdock_info import get_diffdock_info
from .get_dscribe_info import get_dscribe_info
from .get_ec_number_by_entity_id import get_ec_number_by_entity_id
from .get_elephant_info import get_elephant_info
from .get_em_3d_fitting_and_reconstruction_details import (
    get_em_3d_fitting_and_reconstruction_details,
)
from .get_emdb_ids_by_pdb_id import get_emdb_ids_by_pdb_id
from .get_episcanpy_info import get_episcanpy_info
from .get_ete3_info import get_ete3_info
from .get_faiss_info import get_faiss_info
from .get_fanc_info import get_fanc_info
from .get_flask_info import get_flask_info
from .get_flowio_info import get_flowio_info
from .get_flowkit_info import get_flowkit_info
from .get_flowutils_info import get_flowutils_info
from .get_freesasa_info import get_freesasa_info
from .get_galpy_info import get_galpy_info
from .get_gene_name_by_entity_id import get_gene_name_by_entity_id
from .get_geopandas_info import get_geopandas_info
from .get_gget_info import get_gget_info
from .get_googlesearch_python_info import get_googlesearch_python_info
from .get_gseapy_info import get_gseapy_info
from .get_h5py_info import get_h5py_info
from .get_harmony_pytorch_info import get_harmony_pytorch_info
from .get_hmmlearn_info import get_hmmlearn_info
from .get_holoviews_info import get_holoviews_info
from .get_host_organism_by_pdb_id import get_host_organism_by_pdb_id
from .get_htmd_info import get_htmd_info
from .get_igraph_info import get_igraph_info
from .get_imageio_info import get_imageio_info
from .get_imbalanced_learn_info import get_imbalanced_learn_info
from .get_jcvi_info import get_jcvi_info
from .get_joblib_info import get_joblib_info
from .get_joint_associated_diseases_by_HPO_ID_list import (
    get_joint_associated_diseases_by_HPO_ID_list,
)
from .get_khmer_info import get_khmer_info
from .get_kipoiseq_info import get_kipoiseq_info
from .get_lifelines_info import get_lifelines_info
from .get_ligand_bond_count_by_pdb_id import get_ligand_bond_count_by_pdb_id
from .get_ligand_smiles_by_chem_comp_id import get_ligand_smiles_by_chem_comp_id
from .get_lightgbm_info import get_lightgbm_info
from .get_loompy_info import get_loompy_info
from .get_mageck_info import get_mageck_info
from .get_matplotlib_info import get_matplotlib_info
from .get_mdanalysis_info import get_mdanalysis_info
from .get_mdtraj_info import get_mdtraj_info
from .get_mne_info import get_mne_info
from .get_molfeat_info import get_molfeat_info
from .get_molvs_info import get_molvs_info
from .get_mordred_info import get_mordred_info
from .get_msprime_info import get_msprime_info
from .get_mudata_info import get_mudata_info
from .get_mutation_annotations_by_pdb_id import get_mutation_annotations_by_pdb_id
from .get_neo_info import get_neo_info
from .get_netcdf4_info import get_netcdf4_info
from .get_networkx_info import get_networkx_info
from .get_nglview_info import get_nglview_info
from .get_nilearn_info import get_nilearn_info
from .get_numba_info import get_numba_info
from .get_numpy_info import get_numpy_info
from .get_oligosaccharide_descriptors_by_entity_id import (
    get_oligosaccharide_descriptors_by_entity_id,
)
from .get_openbabel_info import get_openbabel_info
from .get_openchem_info import get_openchem_info
from .get_opencv_info import get_opencv_info
from .get_openmm_info import get_openmm_info
from .get_optlang_info import get_optlang_info
from .get_optuna_info import get_optuna_info
from .get_palantir_info import get_palantir_info
from .get_pandas_info import get_pandas_info
from .get_patsy_info import get_patsy_info
from .get_pdbfixer_info import get_pdbfixer_info
from .get_phenotype_by_HPO_ID import get_phenotype_by_HPO_ID
from .get_pillow_info import get_pillow_info
from .get_plantcv_info import get_plantcv_info
from .get_plip_info import get_plip_info
from .get_plotly_info import get_plotly_info
from .get_poliastro_info import get_poliastro_info
from .get_polymer_entity_annotations import get_polymer_entity_annotations
from .get_polymer_entity_count_by_pdb_id import get_polymer_entity_count_by_pdb_id
from .get_polymer_entity_ids_by_pdb_id import get_polymer_entity_ids_by_pdb_id
from .get_polymer_entity_type_by_entity_id import get_polymer_entity_type_by_entity_id
from .get_polymer_molecular_weight_by_entity_id import (
    get_polymer_molecular_weight_by_entity_id,
)
from .get_poretools_info import get_poretools_info
from .get_prody_info import get_prody_info
from .get_protein_classification_by_pdb_id import get_protein_classification_by_pdb_id
from .get_protein_metadata_by_pdb_id import get_protein_metadata_by_pdb_id
from .get_pubchempy_info import get_pubchempy_info
from .get_pybedtools_info import get_pybedtools_info
from .get_pybigwig_info import get_pybigwig_info
from .get_pydeseq2_info import get_pydeseq2_info
from .get_pyensembl_info import get_pyensembl_info
from .get_pyephem_info import get_pyephem_info
from .get_pyfaidx_info import get_pyfaidx_info
from .get_pyfasta_info import get_pyfasta_info
from .get_pykalman_info import get_pykalman_info
from .get_pyliftover_info import get_pyliftover_info
from .get_pymassspec_info import get_pymassspec_info
from .get_pymed_info import get_pymed_info
from .get_pymzml_info import get_pymzml_info
from .get_pypdf2_info import get_pypdf2_info
from .get_pyranges_info import get_pyranges_info
from .get_pyrosetta_info import get_pyrosetta_info
from .get_pysam_info import get_pysam_info
from .get_pyscenic_info import get_pyscenic_info
from .get_pyscf_info import get_pyscf_info
from .get_pyscreener_info import get_pyscreener_info
from .get_pytdc_info import get_pytdc_info
from .get_python_libsbml_info import get_python_libsbml_info
from .get_pytorch_info import get_pytorch_info
from .get_pyvcf_info import get_pyvcf_info
from .get_pyvis_info import get_pyvis_info
from .get_qutip_info import get_qutip_info
from .get_rasterio_info import get_rasterio_info
from .get_rdkit_info import get_rdkit_info
from .get_refinement_resolution_by_pdb_id import get_refinement_resolution_by_pdb_id
from .get_release_deposit_dates_by_pdb_id import get_release_deposit_dates_by_pdb_id
from .get_reportlab_info import get_reportlab_info
from .get_requests_info import get_requests_info
from .get_ruptures_info import get_ruptures_info
from .get_scanorama_info import get_scanorama_info
from .get_scanpy_info import get_scanpy_info
from .get_schnetpack_info import get_schnetpack_info
from .get_scholarly_info import get_scholarly_info
from .get_scikit_bio_info import get_scikit_bio_info
from .get_scikit_image_info import get_scikit_image_info
from .get_scikit_learn_info import get_scikit_learn_info
from .get_scipy_info import get_scipy_info
from .get_scrublet_info import get_scrublet_info
from .get_scvelo_info import get_scvelo_info
from .get_scvi_tools_info import get_scvi_tools_info
from .get_seaborn_info import get_seaborn_info
from .get_sequence_by_pdb_id import get_sequence_by_pdb_id
from .get_sequence_lengths_by_pdb_id import get_sequence_lengths_by_pdb_id
from .get_sequence_positional_features_by_instance_id import (
    get_sequence_positional_features_by_instance_id,
)
from .get_skopt_info import get_skopt_info
from .get_souporcell_info import get_souporcell_info
from .get_source_organism_by_pdb_id import get_source_organism_by_pdb_id
from .get_space_group_by_pdb_id import get_space_group_by_pdb_id
from .get_statsmodels_info import get_statsmodels_info
from .get_structure_determination_software_by_pdb_id import (
    get_structure_determination_software_by_pdb_id,
)
from .get_structure_title_by_pdb_id import get_structure_title_by_pdb_id
from .get_structure_validation_metrics_by_pdb_id import (
    get_structure_validation_metrics_by_pdb_id,
)
from .get_sunpy_info import get_sunpy_info
from .get_sympy_info import get_sympy_info
from .get_target_cofactor_info import get_target_cofactor_info
from .get_taxonomy_by_pdb_id import get_taxonomy_by_pdb_id
from .get_tiledb_info import get_tiledb_info
from .get_tiledbsoma_info import get_tiledbsoma_info
from .get_torch_geometric_info import get_torch_geometric_info
from .get_tqdm_info import get_tqdm_info
from .get_trackpy_info import get_trackpy_info
from .get_tskit_info import get_tskit_info
from .get_umap_learn_info import get_umap_learn_info
from .get_uniprot_accession_by_entity_id import get_uniprot_accession_by_entity_id
from .get_velocyto_info import get_velocyto_info
from .get_viennarna_info import get_viennarna_info
from .get_webpage_text_from_url import get_webpage_text_from_url
from .get_webpage_title import get_webpage_title
from .get_xarray_info import get_xarray_info
from .get_xesmf_info import get_xesmf_info
from .get_xgboost_info import get_xgboost_info
from .get_zarr_info import get_zarr_info
from .gnomad_get_gene_constraints import gnomad_get_gene_constraints
from .gwas_get_association_by_id import gwas_get_association_by_id
from .gwas_get_associations_for_snp import gwas_get_associations_for_snp
from .gwas_get_associations_for_study import gwas_get_associations_for_study
from .gwas_get_associations_for_trait import gwas_get_associations_for_trait
from .gwas_get_snp_by_id import gwas_get_snp_by_id
from .gwas_get_snps_for_gene import gwas_get_snps_for_gene
from .gwas_get_studies_for_trait import gwas_get_studies_for_trait
from .gwas_get_study_by_id import gwas_get_study_by_id
from .gwas_get_variants_for_trait import gwas_get_variants_for_trait
from .gwas_search_associations import gwas_search_associations
from .gwas_search_snps import gwas_search_snps
from .gwas_search_studies import gwas_search_studies
from .humanbase_ppi_analysis import humanbase_ppi_analysis
from .kegg_find_genes import kegg_find_genes
from .kegg_get_gene_info import kegg_get_gene_info
from .kegg_get_pathway_info import kegg_get_pathway_info
from .kegg_list_organisms import kegg_list_organisms
from .kegg_search_pathway import kegg_search_pathway
from .mesh_get_subjects_by_pharmacological_action import (
    mesh_get_subjects_by_pharmacological_action,
)
from .mesh_get_subjects_by_subject_id import mesh_get_subjects_by_subject_id
from .mesh_get_subjects_by_subject_name import mesh_get_subjects_by_subject_name
from .mesh_get_subjects_by_subject_scope_or_definition import (
    mesh_get_subjects_by_subject_scope_or_definition,
)
from .odphp_itemlist import odphp_itemlist
from .odphp_myhealthfinder import odphp_myhealthfinder
from .odphp_outlink_fetch import odphp_outlink_fetch
from .odphp_topicsearch import odphp_topicsearch
from .ols_find_similar_terms import ols_find_similar_terms
from .ols_get_ontology_info import ols_get_ontology_info
from .ols_get_term_ancestors import ols_get_term_ancestors
from .ols_get_term_children import ols_get_term_children
from .ols_get_term_info import ols_get_term_info
from .ols_search_ontologies import ols_search_ontologies
from .ols_search_terms import ols_search_terms
from .open_deep_research_agent import open_deep_research_agent
from .openalex_literature_search import openalex_literature_search
from .python_code_executor import python_code_executor
from .python_script_runner import python_script_runner
from .reactome_disease_target_score import reactome_disease_target_score
from .search_clinical_trials import search_clinical_trials
from .visualize_molecule_2d import visualize_molecule_2d
from .visualize_molecule_3d import visualize_molecule_3d
from .visualize_protein_structure_3d import visualize_protein_structure_3d
from .web_api_documentation_search import web_api_documentation_search
from .web_search import web_search

__all__ = [
    "get_shared_client",
    "reset_shared_client",
    "ADMETAI_predict_BBB_penetrance",
    "ADMETAI_predict_CYP_interactions",
    "ADMETAI_predict_bioavailability",
    "ADMETAI_predict_clearance_distribution",
    "ADMETAI_predict_nuclear_receptor_activity",
    "ADMETAI_predict_physicochemical_properties",
    "ADMETAI_predict_solubility_lipophilicity_hydration",
    "ADMETAI_predict_stress_response",
    "ADMETAI_predict_toxicity",
    "ADMETAnalyzerAgent",
    "AdvancedCodeQualityAnalyzer",
    "AdverseEventICDMapper",
    "AdverseEventPredictionQuestionGenerator",
    "AdverseEventPredictionQuestionGeneratorWithContext",
    "ArXiv_search_papers",
    "ArgumentDescriptionOptimizer",
    "BLAST_nucleotide_search",
    "BLAST_protein_search",
    "BioRxiv_search_preprints",
    "BiomarkerDiscoveryWorkflow",
    "CMA_Guidelines_Search",
    "CORE_search_papers",
    "CallAgent",
    "ChEMBL_search_similar_molecules",
    "ClinicalTrialDesignAgent",
    "CodeQualityAnalyzer",
    "CompoundDiscoveryAgent",
    "ComprehensiveDrugDiscoveryPipeline",
    "Crossref_search_works",
    "DBLP_search_publications",
    "DOAJ_search_articles",
    "DailyMed_get_spl_by_setid",
    "DailyMed_search_spls",
    "DataAnalysisValidityReviewer",
    "DescriptionAnalyzer",
    "DescriptionQualityEvaluator",
    "DiseaseAnalyzerAgent",
    "DomainExpertValidator",
    "DrugInteractionAnalyzerAgent",
    "DrugOptimizationAgent",
    "DrugSafetyAnalyzer",
    "EMDB_get_structure",
    "ENCODE_list_files",
    "ENCODE_search_experiments",
    "EthicalComplianceReviewer",
    "EuropePMC_Guidelines_Search",
    "EuropePMC_search_articles",
    "ExperimentalDesignScorer",
    "FAERS_count_additive_administration_routes",
    "FAERS_count_additive_adverse_reactions",
    "FAERS_count_additive_event_reports_by_country",
    "FAERS_count_additive_reaction_outcomes",
    "FAERS_count_additive_reports_by_reporter_country",
    "FAERS_count_additive_seriousness_classification",
    "FAERS_count_country_by_drug_event",
    "FAERS_count_death_related_by_drug",
    "FAERS_count_drug_routes_by_event",
    "FAERS_count_drugs_by_drug_event",
    "FAERS_count_outcomes_by_drug_event",
    "FAERS_count_patient_age_distribution",
    "FAERS_count_reactions_by_drug_event",
    "FAERS_count_reportercountry_by_drug_event",
    "FAERS_count_seriousness_by_drug_event",
    "FDA_get_abuse_dependence_info_by_drug_name",
    "FDA_get_abuse_info_by_drug_name",
    "FDA_get_accessories_info_by_drug_name",
    "FDA_get_active_ingredient_info_by_drug_name",
    "FDA_get_adverse_reactions_by_drug_name",
    "FDA_get_alarms_by_drug_name",
    "FDA_get_animal_pharmacology_info_by_drug_name",
    "FDA_get_assembly_installation_info_by_drug_name",
    "FDA_get_boxed_warning_info_by_drug_name",
    "FDA_get_brand_name_generic_name",
    "FDA_get_calibration_instructions_by_drug_name",
    "FDA_get_carcinogenic_mutagenic_fertility_by_drug_name",
    "FDA_get_child_safety_info_by_drug_name",
    "FDA_get_clinical_pharmacology_by_drug_name",
    "FDA_get_clinical_studies_info_by_drug_name",
    "FDA_get_contact_for_questions_info_by_drug_name",
    "FDA_get_contraindications_by_drug_name",
    "FDA_get_controlled_substance_DEA_schedule_info_by_drug_name",
    "FDA_get_dear_health_care_provider_letter_info_by_drug_name",
    "FDA_get_dependence_info_by_drug_name",
    "FDA_get_disposal_info_by_drug_name",
    "FDA_get_do_not_use_info_by_drug_name",
    "FDA_get_document_id_by_drug_name",
    "FDA_get_dosage_and_storage_information_by_drug_name",
    "FDA_get_dosage_forms_and_strengths_by_drug_name",
    "FDA_get_drug_generic_name",
    "FDA_get_drug_interactions_by_drug_name",
    "FDA_get_drug_name_by_SPL_ID",
    "FDA_get_drug_name_by_adverse_reaction",
    "FDA_get_drug_name_by_calibration_instructions",
    "FDA_get_drug_name_by_dependence_info",
    "FDA_get_drug_name_by_document_id",
    "FDA_get_drug_name_by_dosage_info",
    "FDA_get_drug_name_by_environmental_warning",
    "FDA_get_drug_name_by_inactive_ingredient",
    "FDA_get_drug_name_by_info_on_conditions_for_doctor_consultation",
    "FDA_get_drug_name_by_labor_and_delivery_info",
    "FDA_get_drug_name_by_microbiology",
    "FDA_get_drug_name_by_other_safety_info",
    "FDA_get_drug_name_by_pharmacodynamics",
    "FDA_get_drug_name_by_pharmacogenomics",
    "FDA_get_drug_name_by_precautions",
    "FDA_get_drug_name_by_pregnancy_or_breastfeeding_info",
    "FDA_get_drug_name_by_principal_display_panel",
    "FDA_get_drug_name_by_reference",
    "FDA_get_drug_name_by_set_id",
    "FDA_get_drug_name_by_stop_use_info",
    "FDA_get_drug_name_by_storage_and_handling_info",
    "FDA_get_drug_name_by_warnings",
    "FDA_get_drug_name_from_patient_package_insert",
    "FDA_get_drug_names_by_abuse_dependence_info",
    "FDA_get_drug_names_by_abuse_info",
    "FDA_get_drug_names_by_accessories",
    "FDA_get_drug_names_by_active_ingredient",
    "FDA_get_drug_names_by_alarm",
    "FDA_get_drug_names_by_animal_pharmacology_info",
    "FDA_get_drug_names_by_application_number_NDC_number",
    "FDA_get_drug_names_by_assembly_installation_info",
    "FDA_get_drug_names_by_boxed_warning",
    "FDA_get_drug_names_by_child_safety_info",
    "FDA_get_drug_names_by_clinical_pharmacology",
    "FDA_get_drug_names_by_clinical_studies",
    "FDA_get_drug_names_by_consulting_doctor_pharmacist_info",
    "FDA_get_drug_names_by_contraindications",
    "FDA_get_drug_names_by_controlled_substance_DEA_schedule",
    "FDA_get_drug_names_by_dear_health_care_provider_letter_info",
    "FDA_get_drug_names_by_disposal_info",
    "FDA_get_drug_names_by_dosage_forms_and_strengths_info",
    "FDA_get_drug_names_by_drug_interactions",
    "FDA_get_drug_names_by_effective_time",
    "FDA_get_drug_names_by_food_safety_warnings",
    "FDA_get_drug_names_by_general_precautions",
    "FDA_get_drug_names_by_geriatric_use",
    "FDA_get_drug_names_by_health_claim",
    "FDA_get_drug_names_by_indication",
    "FDA_get_drug_names_by_info_for_nursing_mothers",
    "FDA_get_drug_names_by_information_for_owners_or_caregivers",
    "FDA_get_drug_names_by_ingredient",
    "FDA_get_drug_names_by_instructions_for_use",
    "FDA_get_drug_names_by_lab_test_interference",
    "FDA_get_drug_names_by_lab_tests",
    "FDA_get_drug_names_by_mechanism_of_action",
    "FDA_get_drug_names_by_medication_guide",
    "FDA_get_drug_names_by_nonclinical_toxicology_info",
    "FDA_get_drug_names_by_nonteratogenic_effects",
    "FDA_get_drug_names_by_overdosage_info",
    "FDA_get_drug_names_by_pediatric_use",
    "FDA_get_drug_names_by_pharmacokinetics",
    "FDA_get_drug_names_by_population_use",
    "FDA_get_drug_names_by_pregnancy_effects_info",
    "FDA_get_drug_names_by_residue_warning",
    "FDA_get_drug_names_by_risk",
    "FDA_get_drug_names_by_route",
    "FDA_get_drug_names_by_safe_handling_warning",
    "FDA_get_drug_names_by_safety_summary",
    "FDA_get_drug_names_by_spl_indexing_data_elements",
    "FDA_get_drug_names_by_teratogenic_effects",
    "FDA_get_drug_names_by_user_safety_warning",
    "FDA_get_drug_names_by_warnings_and_cautions",
    "FDA_get_drugs_by_carcinogenic_mutagenic_fertility",
    "FDA_get_effective_time_by_drug_name",
    "FDA_get_environmental_warning_by_drug_name",
    "FDA_get_general_precautions_by_drug_name",
    "FDA_get_geriatric_use_info_by_drug_name",
    "FDA_get_health_claims_by_drug_name",
    "FDA_get_inactive_ingredient_info_by_drug_name",
    "FDA_get_indications_by_drug_name",
    "FDA_get_info_for_nursing_mothers_by_drug_name",
    "FDA_get_info_for_patients_by_drug_name",
    "FDA_get_info_on_conditions_for_doctor_consultation_by_drug_name",
    "FDA_get_info_on_consulting_doctor_pharmacist_by_drug_name",
    "FDA_get_information_for_owners_or_caregivers_by_drug_name",
    "FDA_get_ingredients_by_drug_name",
    "FDA_get_instructions_for_use_by_drug_name",
    "FDA_get_lab_test_interference_info_by_drug_name",
    "FDA_get_lab_tests_by_drug_name",
    "FDA_get_labor_and_delivery_info_by_drug_name",
    "FDA_get_manufacturer_name_NDC_number_by_drug_name",
    "FDA_get_mechanism_of_action_by_drug_name",
    "FDA_get_medication_guide_info_by_drug_name",
    "FDA_get_microbiology_info_by_drug_name",
    "FDA_get_nonclinical_toxicology_info_by_drug_name",
    "FDA_get_nonteratogenic_effects_by_drug_name",
    "FDA_get_other_safety_info_by_drug_name",
    "FDA_get_overdosage_info_by_drug_name",
    "FDA_get_patient_package_insert_from_drug_name",
    "FDA_get_pediatric_use_info_by_drug_name",
    "FDA_get_pharmacodynamics_by_drug_name",
    "FDA_get_pharmacogenomics_info_by_drug_name",
    "FDA_get_pharmacokinetics_by_drug_name",
    "FDA_get_population_use_info_by_drug_name",
    "FDA_get_precautions_by_drug_name",
    "FDA_get_pregnancy_effects_info_by_drug_name",
    "FDA_get_pregnancy_or_breastfeeding_info_by_drug_name",
    "FDA_get_principal_display_panel_by_drug_name",
    "FDA_get_purpose_info_by_drug_name",
    "FDA_get_recent_changes_by_drug_name",
    "FDA_get_reference_info_by_drug_name",
    "FDA_get_residue_warning_by_drug_name",
    "FDA_get_risk_info_by_drug_name",
    "FDA_get_route_info_by_drug_name",
    "FDA_get_safe_handling_warnings_by_drug_name",
    "FDA_get_safety_summary_by_drug_name",
    "FDA_get_spl_indexing_data_elements_by_drug_name",
    "FDA_get_spl_unclassified_section_by_drug_name",
    "FDA_get_stop_use_info_by_drug_name",
    "FDA_get_storage_and_handling_info_by_drug_name",
    "FDA_get_teratogenic_effects_by_drug_name",
    "FDA_get_user_safety_warning_by_drug_names",
    "FDA_get_warnings_and_cautions_by_drug_name",
    "FDA_get_warnings_by_drug_name",
    "FDA_get_when_using_info",
    "FDA_retrieve_device_use_by_drug_name",
    "FDA_retrieve_drug_name_by_device_use",
    "FDA_retrieve_drug_names_by_patient_medication_info",
    "FDA_retrieve_patient_medication_info_by_drug_name",
    "Fatcat_search_scholar",
    "Finish",
    "GBIF_search_occurrences",
    "GBIF_search_species",
    "GDC_list_files",
    "GDC_search_cases",
    "GIN_Guidelines_Search",
    "GO_get_annotations_for_gene",
    "GO_get_genes_for_term",
    "GO_get_term_by_id",
    "GO_get_term_details",
    "GO_search_terms",
    "GTEx_get_expression_summary",
    "GTEx_query_eqtl",
    "GWAS_search_associations_by_gene",
    "GtoPdb_get_targets",
    "HAL_search_archive",
    "HPA_get_biological_processes_by_gene",
    "HPA_get_cancer_prognostics_by_gene",
    "HPA_get_comparative_expression_by_gene_and_cellline",
    "HPA_get_comprehensive_gene_details_by_ensembl_id",
    "HPA_get_contextual_biological_process_analysis",
    "HPA_get_disease_expression_by_gene_tissue_disease",
    "HPA_get_gene_basic_info_by_ensembl_id",
    "HPA_get_gene_tsv_data_by_ensembl_id",
    "HPA_get_protein_interactions_by_gene",
    "HPA_get_rna_expression_by_source",
    "HPA_get_rna_expression_in_specific_tissues",
    "HPA_get_subcellular_location",
    "HPA_search_genes_by_query",
    "HypothesisGenerator",
    "InterPro_get_domain_details",
    "InterPro_get_protein_domains",
    "InterPro_search_domains",
    "JASPAR_get_transcription_factors",
    "LabelGenerator",
    "LiteratureContextReviewer",
    "LiteratureSearchTool",
    "LiteratureSynthesisAgent",
    "MGnify_list_analyses",
    "MGnify_search_studies",
    "MPD_get_phenotype_data",
    "MedRxiv_search_preprints",
    "MedicalLiteratureReviewer",
    "MedicalTermNormalizer",
    "MedlinePlus_connect_lookup_by_code",
    "MedlinePlus_get_genetics_condition_by_name",
    "MedlinePlus_get_genetics_gene_by_name",
    "MedlinePlus_get_genetics_index",
    "MedlinePlus_search_topics_by_keyword",
    "MethodologyRigorReviewer",
    "NICE_Clinical_Guidelines_Search",
    "NICE_Guideline_Full_Text",
    "NoveltySignificanceReviewer",
    "OBIS_search_occurrences",
    "OBIS_search_taxa",
    "OSF_search_preprints",
    "OSL_get_efo_id_by_disease_name",
    "OpenAIRE_search_publications",
    "OpenAlex_Guidelines_Search",
    "OpenTargets_drug_pharmacogenomics_data",
    "OpenTargets_get_approved_indications_by_drug_chemblId",
    "OpenTargets_get_associated_diseases_by_drug_chemblId",
    "OpenTargets_get_associated_drugs_by_disease_efoId",
    "OpenTargets_get_associated_drugs_by_target_ensemblID",
    "OpenTargets_get_associated_phenotypes_by_disease_efoId",
    "OpenTargets_get_associated_targets_by_disease_efoId",
    "OpenTargets_get_associated_targets_by_drug_chemblId",
    "OpenTargets_get_biological_mouse_models_by_ensemblID",
    "OpenTargets_get_chemical_probes_by_target_ensemblID",
    "OpenTargets_get_disease_ancestors_parents_by_efoId",
    "OpenTargets_get_disease_descendants_children_by_efoId",
    "OpenTargets_get_disease_description_by_efoId",
    "OpenTargets_get_disease_id_description_by_name",
    "OpenTargets_get_disease_ids_by_efoId",
    "OpenTargets_get_disease_ids_by_name",
    "OpenTargets_get_disease_locations_by_efoId",
    "OpenTargets_get_disease_synonyms_by_efoId",
    "OpenTargets_get_disease_therapeutic_areas_by_efoId",
    "OpenTargets_get_diseases_phenotypes_by_target_ensembl",
    "OpenTargets_get_drug_adverse_events_by_chemblId",
    "OpenTargets_get_drug_approval_status_by_chemblId",
    "OpenTargets_get_drug_chembId_by_generic_name",
    "OpenTargets_get_drug_description_by_chemblId",
    "OpenTargets_get_drug_id_description_by_name",
    "OpenTargets_get_drug_indications_by_chemblId",
    "OpenTargets_get_drug_mechanisms_of_action_by_chemblId",
    "OpenTargets_get_drug_synonyms_by_chemblId",
    "OpenTargets_get_drug_trade_names_by_chemblId",
    "OpenTargets_get_drug_warnings_by_chemblId",
    "OpenTargets_get_drug_withdrawn_blackbox_status_by_chemblId",
    "OpenTargets_get_gene_ontology_terms_by_goID",
    "OpenTargets_get_known_drugs_by_drug_chemblId",
    "OpenTargets_get_parent_child_molecules_by_drug_chembl_ID",
    "OpenTargets_get_publications_by_disease_efoId",
    "OpenTargets_get_publications_by_drug_chemblId",
    "OpenTargets_get_publications_by_target_ensemblID",
    "OpenTargets_get_similar_entities_by_disease_efoId",
    "OpenTargets_get_similar_entities_by_drug_chemblId",
    "OpenTargets_get_similar_entities_by_target_ensemblID",
    "OpenTargets_get_target_classes_by_ensemblID",
    "OpenTargets_get_target_constraint_info_by_ensemblID",
    "OpenTargets_get_target_enabling_packages_by_ensemblID",
    "OpenTargets_get_target_gene_ontology_by_ensemblID",
    "OpenTargets_get_target_genomic_location_by_ensemblID",
    "OpenTargets_get_target_homologues_by_ensemblID",
    "OpenTargets_get_target_id_description_by_name",
    "OpenTargets_get_target_interactions_by_ensemblID",
    "OpenTargets_get_target_safety_profile_by_ensemblID",
    "OpenTargets_get_target_subcellular_locations_by_ensemblID",
    "OpenTargets_get_target_synonyms_by_ensemblID",
    "OpenTargets_get_target_tractability_by_ensemblID",
    "OpenTargets_map_any_disease_id_to_all_other_ids",
    "OpenTargets_multi_entity_search_by_query_string",
    "OpenTargets_search_category_counts_by_query_string",
    "OpenTargets_target_disease_evidence",
    "OutputSummarizationComposer",
    "PMC_search_papers",
    "PRIDE_search_proteomics",
    "PackageAnalyzer",
    "Paleobiology_get_fossils",
    "ProtocolOptimizer",
    "PubChem_get_CID_by_SMILES",
    "PubChem_get_CID_by_compound_name",
    "PubChem_get_associated_patents_by_CID",
    "PubChem_get_compound_2D_image_by_CID",
    "PubChem_get_compound_properties_by_CID",
    "PubChem_get_compound_synonyms_by_CID",
    "PubChem_get_compound_xrefs_by_CID",
    "PubChem_search_compounds_by_similarity",
    "PubChem_search_compounds_by_substructure",
    "PubMed_Guidelines_Search",
    "PubMed_search_articles",
    "PubTator3_EntityAutocomplete",
    "PubTator3_LiteratureSearch",
    "PyPIPackageInspector",
    "QuestionRephraser",
    "RNAcentral_get_by_accession",
    "RNAcentral_search",
    "ReMap_get_transcription_factor_binding",
    "Reactome_get_pathway_reactions",
    "ReferenceInfoAnalyzer",
    "RegulomeDB_query_variant",
    "ReproducibilityTransparencyReviewer",
    "ResultsInterpretationReviewer",
    "SCREEN_get_regulatory_elements",
    "ScientificTextSummarizer",
    "SemanticScholar_search_papers",
    "TRIP_Database_Guidelines_Search",
    "TestCaseGenerator",
    "TestResultsAnalyzer",
    "ToolCompatibilityAnalyzer",
    "ToolDescriptionOptimizer",
    "ToolDiscover",
    "ToolGraphComposer",
    "ToolGraphGenerationPipeline",
    "ToolMetadataGenerationPipeline",
    "ToolMetadataGenerator",
    "ToolMetadataStandardizer",
    "ToolOutputSummarizer",
    "ToolQualityEvaluator",
    "ToolRelationshipDetector",
    "Tool_Finder",
    "Tool_Finder_Keyword",
    "Tool_Finder_LLM",
    "Tool_RAG",
    "UniProt_get_alternative_names_by_accession",
    "UniProt_get_disease_variants_by_accession",
    "UniProt_get_entry_by_accession",
    "UniProt_get_function_by_accession",
    "UniProt_get_isoform_ids_by_accession",
    "UniProt_get_organism_by_accession",
    "UniProt_get_ptm_processing_by_accession",
    "UniProt_get_recommended_name_by_accession",
    "UniProt_get_sequence_by_accession",
    "UniProt_get_subcellular_location_by_accession",
    "UniProt_id_mapping",
    "UniProt_search",
    "UnifiedToolGenerator",
    "Unpaywall_check_oa_status",
    "WHO_Guideline_Full_Text",
    "WHO_Guidelines_Search",
    "WikiPathways_get_pathway",
    "WikiPathways_search",
    "Wikidata_SPARQL_query",
    "WoRMS_search_species",
    "WritingPresentationReviewer",
    "XMLToolOptimizer",
    "Zenodo_search_records",
    "advanced_literature_search_agent",
    "alphafold_get_annotations",
    "alphafold_get_prediction",
    "alphafold_get_summary",
    "cBioPortal_get_cancer_studies",
    "cBioPortal_get_mutations",
    "call_agentic_human",
    "cancer_biomarkers_disease_target_score",
    "cancer_gene_census_disease_target_score",
    "cellosaurus_get_cell_line_info",
    "cellosaurus_query_converter",
    "cellosaurus_search_cell_lines",
    "chembl_disease_target_score",
    "clinvar_get_clinical_significance",
    "clinvar_get_variant_details",
    "clinvar_search_variants",
    "convert_to_markdown",
    "dbsnp_get_frequencies",
    "dbsnp_get_variant_by_rsid",
    "dbsnp_search_by_gene",
    "dict_search",
    "dili_search",
    "diqt_search",
    "disease_target_score",
    "download_binary_file",
    "download_file",
    "download_text_content",
    "drugbank_filter_drugs_by_name",
    "drugbank_full_search",
    "drugbank_get_drug_basic_info_by_drug_name_or_drugbank_id",
    "drugbank_get_drug_chemistry_by_drug_name_or_drugbank_id",
    "drugbank_get_drug_desc_pharmacology_by_moa",
    "drugbank_get_drug_interactions_by_drug_name_or_drugbank_id",
    "drugbank_get_drug_name_and_description_by_indication",
    "drugbank_get_drug_name_and_description_by_pathway_name",
    "drugbank_get_drug_name_and_description_by_target_name",
    "drugbank_get_drug_products_by_name_or_drugbank_id",
    "drugbank_get_drug_references_by_drug_name_or_drugbank_id",
    "drugbank_get_indications_by_drug_name_or_drugbank_id",
    "drugbank_get_pathways_reactions_by_drug_or_id",
    "drugbank_get_pharmacology_by_drug_name_or_drugbank_id",
    "drugbank_get_safety_by_drug_name_or_drugbank_id",
    "drugbank_get_targets_by_drug_name_or_drugbank_id",
    "drugbank_links_search",
    "drugbank_vocab_filter",
    "drugbank_vocab_search",
    "dynamic_package_discovery",
    "embedding_database_add",
    "embedding_database_create",
    "embedding_database_load",
    "embedding_database_search",
    "embedding_sync_download",
    "embedding_sync_upload",
    "enrichr_gene_enrichment_analysis",
    "ensembl_get_sequence",
    "ensembl_get_variants",
    "ensembl_lookup_gene",
    "europepmc_disease_target_score",
    "eva_disease_target_score",
    "eva_somatic_disease_target_score",
    "expression_atlas_disease_target_score",
    "extract_clinical_trial_adverse_events",
    "extract_clinical_trial_outcomes",
    "genomics_england_disease_target_score",
    "geo_get_dataset_info",
    "geo_get_sample_info",
    "geo_search_datasets",
    "get_HPO_ID_by_phenotype",
    "get_albumentations_info",
    "get_altair_info",
    "get_anndata_info",
    "get_arboreto_info",
    "get_arxiv_info",
    "get_ase_info",
    "get_assembly_info_by_pdb_id",
    "get_assembly_summary",
    "get_astropy_info",
    "get_binding_affinity_by_pdb_id",
    "get_biopandas_info",
    "get_biopython_info",
    "get_bioservices_info",
    "get_biotite_info",
    "get_bokeh_info",
    "get_brian2_info",
    "get_cartopy_info",
    "get_catboost_info",
    "get_cellpose_info",
    "get_cellrank_info",
    "get_cellxgene_census_info",
    "get_cftime_info",
    "get_chem_comp_audit_info",
    "get_chem_comp_charge_and_ambiguity",
    "get_chembl_webresource_client_info",
    "get_citation_info_by_pdb_id",
    "get_clair3_info",
    "get_clinical_trial_conditions_and_interventions",
    "get_clinical_trial_descriptions",
    "get_clinical_trial_eligibility_criteria",
    "get_clinical_trial_locations",
    "get_clinical_trial_outcome_measures",
    "get_clinical_trial_references",
    "get_clinical_trial_status_and_dates",
    "get_cobra_info",
    "get_cobrapy_info",
    "get_cooler_info",
    "get_core_refinement_statistics",
    "get_cryosparc_tools_info",
    "get_crystal_growth_conditions_by_pdb_id",
    "get_crystallization_ph_by_pdb_id",
    "get_crystallographic_properties_by_pdb_id",
    "get_cupy_info",
    "get_cyvcf2_info",
    "get_dask_info",
    "get_datamol_info",
    "get_datashader_info",
    "get_deepchem_info",
    "get_deeppurpose_info",
    "get_deeptools_info",
    "get_deepxde_info",
    "get_dendropy_info",
    "get_descriptastorus_info",
    "get_diffdock_info",
    "get_dscribe_info",
    "get_ec_number_by_entity_id",
    "get_elephant_info",
    "get_em_3d_fitting_and_reconstruction_details",
    "get_emdb_ids_by_pdb_id",
    "get_episcanpy_info",
    "get_ete3_info",
    "get_faiss_info",
    "get_fanc_info",
    "get_flask_info",
    "get_flowio_info",
    "get_flowkit_info",
    "get_flowutils_info",
    "get_freesasa_info",
    "get_galpy_info",
    "get_gene_name_by_entity_id",
    "get_geopandas_info",
    "get_gget_info",
    "get_googlesearch_python_info",
    "get_gseapy_info",
    "get_h5py_info",
    "get_harmony_pytorch_info",
    "get_hmmlearn_info",
    "get_holoviews_info",
    "get_host_organism_by_pdb_id",
    "get_htmd_info",
    "get_igraph_info",
    "get_imageio_info",
    "get_imbalanced_learn_info",
    "get_jcvi_info",
    "get_joblib_info",
    "get_joint_associated_diseases_by_HPO_ID_list",
    "get_khmer_info",
    "get_kipoiseq_info",
    "get_lifelines_info",
    "get_ligand_bond_count_by_pdb_id",
    "get_ligand_smiles_by_chem_comp_id",
    "get_lightgbm_info",
    "get_loompy_info",
    "get_mageck_info",
    "get_matplotlib_info",
    "get_mdanalysis_info",
    "get_mdtraj_info",
    "get_mne_info",
    "get_molfeat_info",
    "get_molvs_info",
    "get_mordred_info",
    "get_msprime_info",
    "get_mudata_info",
    "get_mutation_annotations_by_pdb_id",
    "get_neo_info",
    "get_netcdf4_info",
    "get_networkx_info",
    "get_nglview_info",
    "get_nilearn_info",
    "get_numba_info",
    "get_numpy_info",
    "get_oligosaccharide_descriptors_by_entity_id",
    "get_openbabel_info",
    "get_openchem_info",
    "get_opencv_info",
    "get_openmm_info",
    "get_optlang_info",
    "get_optuna_info",
    "get_palantir_info",
    "get_pandas_info",
    "get_patsy_info",
    "get_pdbfixer_info",
    "get_phenotype_by_HPO_ID",
    "get_pillow_info",
    "get_plantcv_info",
    "get_plip_info",
    "get_plotly_info",
    "get_poliastro_info",
    "get_polymer_entity_annotations",
    "get_polymer_entity_count_by_pdb_id",
    "get_polymer_entity_ids_by_pdb_id",
    "get_polymer_entity_type_by_entity_id",
    "get_polymer_molecular_weight_by_entity_id",
    "get_poretools_info",
    "get_prody_info",
    "get_protein_classification_by_pdb_id",
    "get_protein_metadata_by_pdb_id",
    "get_pubchempy_info",
    "get_pybedtools_info",
    "get_pybigwig_info",
    "get_pydeseq2_info",
    "get_pyensembl_info",
    "get_pyephem_info",
    "get_pyfaidx_info",
    "get_pyfasta_info",
    "get_pykalman_info",
    "get_pyliftover_info",
    "get_pymassspec_info",
    "get_pymed_info",
    "get_pymzml_info",
    "get_pypdf2_info",
    "get_pyranges_info",
    "get_pyrosetta_info",
    "get_pysam_info",
    "get_pyscenic_info",
    "get_pyscf_info",
    "get_pyscreener_info",
    "get_pytdc_info",
    "get_python_libsbml_info",
    "get_pytorch_info",
    "get_pyvcf_info",
    "get_pyvis_info",
    "get_qutip_info",
    "get_rasterio_info",
    "get_rdkit_info",
    "get_refinement_resolution_by_pdb_id",
    "get_release_deposit_dates_by_pdb_id",
    "get_reportlab_info",
    "get_requests_info",
    "get_ruptures_info",
    "get_scanorama_info",
    "get_scanpy_info",
    "get_schnetpack_info",
    "get_scholarly_info",
    "get_scikit_bio_info",
    "get_scikit_image_info",
    "get_scikit_learn_info",
    "get_scipy_info",
    "get_scrublet_info",
    "get_scvelo_info",
    "get_scvi_tools_info",
    "get_seaborn_info",
    "get_sequence_by_pdb_id",
    "get_sequence_lengths_by_pdb_id",
    "get_sequence_positional_features_by_instance_id",
    "get_skopt_info",
    "get_souporcell_info",
    "get_source_organism_by_pdb_id",
    "get_space_group_by_pdb_id",
    "get_statsmodels_info",
    "get_structure_determination_software_by_pdb_id",
    "get_structure_title_by_pdb_id",
    "get_structure_validation_metrics_by_pdb_id",
    "get_sunpy_info",
    "get_sympy_info",
    "get_target_cofactor_info",
    "get_taxonomy_by_pdb_id",
    "get_tiledb_info",
    "get_tiledbsoma_info",
    "get_torch_geometric_info",
    "get_tqdm_info",
    "get_trackpy_info",
    "get_tskit_info",
    "get_umap_learn_info",
    "get_uniprot_accession_by_entity_id",
    "get_velocyto_info",
    "get_viennarna_info",
    "get_webpage_text_from_url",
    "get_webpage_title",
    "get_xarray_info",
    "get_xesmf_info",
    "get_xgboost_info",
    "get_zarr_info",
    "gnomad_get_gene_constraints",
    "gwas_get_association_by_id",
    "gwas_get_associations_for_snp",
    "gwas_get_associations_for_study",
    "gwas_get_associations_for_trait",
    "gwas_get_snp_by_id",
    "gwas_get_snps_for_gene",
    "gwas_get_studies_for_trait",
    "gwas_get_study_by_id",
    "gwas_get_variants_for_trait",
    "gwas_search_associations",
    "gwas_search_snps",
    "gwas_search_studies",
    "humanbase_ppi_analysis",
    "kegg_find_genes",
    "kegg_get_gene_info",
    "kegg_get_pathway_info",
    "kegg_list_organisms",
    "kegg_search_pathway",
    "mesh_get_subjects_by_pharmacological_action",
    "mesh_get_subjects_by_subject_id",
    "mesh_get_subjects_by_subject_name",
    "mesh_get_subjects_by_subject_scope_or_definition",
    "odphp_itemlist",
    "odphp_myhealthfinder",
    "odphp_outlink_fetch",
    "odphp_topicsearch",
    "ols_find_similar_terms",
    "ols_get_ontology_info",
    "ols_get_term_ancestors",
    "ols_get_term_children",
    "ols_get_term_info",
    "ols_search_ontologies",
    "ols_search_terms",
    "open_deep_research_agent",
    "openalex_literature_search",
    "python_code_executor",
    "python_script_runner",
    "reactome_disease_target_score",
    "search_clinical_trials",
    "visualize_molecule_2d",
    "visualize_molecule_3d",
    "visualize_protein_structure_3d",
    "web_api_documentation_search",
    "web_search",
]



"""
Protein Structure 3D Visualization Tool
=======================================

Tool for visualizing 3D protein structures using py3Dmol.
Supports PDB IDs, PDB file content, and various visualization styles.
"""

import requests
from typing import Any, Dict
from .visualization_tool import VisualizationTool
from .tool_registry import register_tool


@register_tool("ProteinStructure3DTool")
class ProteinStructure3DTool(VisualizationTool):
    """Tool for visualizing 3D protein structures using py3Dmol."""

    def run(self, arguments: Dict[str, Any]) -> Dict[str, Any]:
        """Generate 3D protein structure visualization."""
        try:
            import py3Dmol

            # Extract parameters
            pdb_id = arguments.get("pdb_id")
            pdb_content = arguments.get("pdb_content")
            style = arguments.get("style", "cartoon")
            color_scheme = arguments.get("color_scheme", "spectrum")
            width = arguments.get("width", self.default_width)
            height = arguments.get("height", self.default_height)
            show_sidechains = arguments.get("show_sidechains", False)
            show_surface = arguments.get("show_surface", False)

            # Get PDB content
            if pdb_content:
                pdb_data = pdb_content
            elif pdb_id:
                pdb_data = self._fetch_pdb_content(pdb_id)
            else:
                return self.create_error_response(
                    "Either pdb_id or pdb_content must be provided"
                )

            # Create py3Dmol viewer
            viewer = py3Dmol.view(width=width, height=height)
            viewer.addModel(pdb_data, "pdb")

            # Apply visualization style
            if style == "cartoon":
                viewer.setStyle({"cartoon": {"color": color_scheme}})
            elif style == "stick":
                viewer.setStyle({"stick": {"color": color_scheme}})
            elif style == "sphere":
                viewer.setStyle({"sphere": {"color": color_scheme}})
            elif style == "line":
                viewer.setStyle({"line": {"color": color_scheme}})
            else:
                viewer.setStyle({"cartoon": {"color": color_scheme}})

            # Add sidechains if requested
            if show_sidechains:
                resn_list = [
                    "ALA",
                    "ARG",
                    "ASN",
                    "ASP",
                    "CYS",
                    "GLN",
                    "GLU",
                    "GLY",
                    "HIS",
                    "ILE",
                    "LEU",
                    "LYS",
                    "MET",
                    "PHE",
                    "PRO",
                    "SER",
                    "THR",
                    "TRP",
                    "TYR",
                    "VAL",
                ]
                viewer.addStyle(
                    {"stick": {"radius": 0.1}}, {"and": [{"resn": resn_list}]}
                )

            # Add surface if requested
            if show_surface:
                viewer.addSurface(py3Dmol.VDW, {"opacity": 0.7, "color": "white"})

            # Zoom to fit
            viewer.zoomTo()

            # Generate HTML with modern UI
            viewer_html = viewer._make_html()

            # Create control panel
            control_panel = self._create_control_panel(style, color_scheme)

            # Create toolbar
            toolbar = self._create_toolbar()

            # Create info cards
            info_cards = self._create_protein_info_cards(pdb_id, pdb_data)

            # Generate modern HTML
            html_content = self.create_py3dmol_html(
                viewer_html,
                width,
                height,
                title=f"Protein Structure: {pdb_id or 'Custom PDB'}",
                control_panel=control_panel,
                toolbar=toolbar,
                info_cards=info_cards,
            )

            # Add JavaScript controls
            html_content = html_content.replace(
                "</body>", f"{self.add_3d_controls_script()}</body>"
            )

            # Prepare metadata
            metadata = {
                "width": width,
                "height": height,
                "style": style,
                "color_scheme": color_scheme,
                "pdb_id": pdb_id,
                "show_sidechains": show_sidechains,
                "show_surface": show_surface,
            }

            return self.create_visualization_response(
                html_content=html_content,
                viz_type="protein_structure_3d",
                data={
                    "pdb_content": (
                        pdb_data[:500] + "..." if len(pdb_data) > 500 else pdb_data
                    )
                },
                metadata=metadata,
            )

        except ImportError:
            return self.create_error_response(
                "py3Dmol is not installed. Please install it with: "
                "pip install py3Dmol",
                "MissingDependency",
            )
        except Exception as e:
            return self.create_error_response(
                f"Failed to create protein visualization: {str(e)}"
            )

    def _fetch_pdb_content(self, pdb_id: str) -> str:
        """Fetch PDB content from RCSB PDB database."""
        try:
            url = f"https://files.rcsb.org/view/{pdb_id.upper()}.pdb"
            response = requests.get(url, timeout=30)
            response.raise_for_status()
            return response.text
        except requests.RequestException as e:
            raise ValueError(f"Failed to fetch PDB file for {pdb_id}: {str(e)}")

    def _create_control_panel(self, current_style: str, current_color: str) -> str:
        """Create floating control panel HTML."""
        return f"""
        <div class="control-panel">
            <div class="control-group">
                <label class="control-label">Style</label>
                <select class="control-select" id="styleSelect" onchange="changeStyle()">
                    <option value="cartoon" {'selected' if current_style == 'cartoon' else ''}>Cartoon</option>
                    <option value="stick" {'selected' if current_style == 'stick' else ''}>Stick</option>
                    <option value="sphere" {'selected' if current_style == 'sphere' else ''}>Sphere</option>
                    <option value="line" {'selected' if current_style == 'line' else ''}>Line</option>
                    <option value="surface" {'selected' if current_style == 'surface' else ''}>Surface</option>
                </select>
            </div>
            <div class="control-group">
                <label class="control-label">Color Scheme</label>
                <select class="control-select" id="colorSelect" onchange="changeColor()">
                    <option value="spectrum" {'selected' if current_color == 'spectrum' else ''}>Spectrum</option>
                    <option value="rainbow" {'selected' if current_color == 'rainbow' else ''}>Rainbow</option>
                    <option value="ss" {'selected' if current_color == 'ss' else ''}>Secondary Structure</option>
                    <option value="chain" {'selected' if current_color == 'chain' else ''}>Chain</option>
                    <option value="elem" {'selected' if current_color == 'elem' else ''}>Element</option>
                </select>
            </div>
            <div class="control-group">
                <label class="control-label">Background</label>
                <select class="control-select" id="bgSelect" onchange="changeBackground()">
                    <option value="white" selected>White</option>
                    <option value="black">Black</option>
                    <option value="gray">Gray</option>
                </select>
            </div>
        </div>
        """

    def _create_toolbar(self) -> str:
        """Create bottom toolbar HTML."""
        return """
        <div class="toolbar">
            <button class="btn" onclick="resetView()">Reset View</button>
            <button class="btn btn-secondary" onclick="downloadScreenshot()">Screenshot</button>
            <button class="btn btn-outline" onclick="toggleFullscreen()">Fullscreen</button>
        </div>
        """

    def _create_protein_info_cards(self, pdb_id: str, pdb_data: str) -> str:
        """Create protein information cards."""
        # Parse basic PDB info
        lines = pdb_data.split("\n")
        title = "Unknown Protein"
        organism = "Unknown"
        resolution = "N/A"
        method = "Unknown"
        residue_count = 0
        atom_count = 0

        for line in lines:
            if line.startswith("TITLE"):
                title = line[10:].strip()
            elif line.startswith("SOURCE"):
                if "ORGANISM_SCIENTIFIC" in line:
                    organism = (
                        line.split("ORGANISM_SCIENTIFIC:")[1].split(";")[0].strip()
                    )
            elif line.startswith("REMARK   2 RESOLUTION"):
                resolution = line.split("RESOLUTION.")[1].split("ANGSTROMS")[0].strip()
            elif line.startswith("EXPDTA"):
                method = line[10:].strip()
            elif line.startswith("ATOM"):
                atom_count += 1
                if atom_count == 1:
                    residue_count = 1
                elif line[22:26].strip() != lines[atom_count - 2][22:26].strip():
                    residue_count += 1

        return f"""
        <div class="card">
            <h3 class="card-title">
                <svg class="card-icon" viewBox="0 0 24 24">
                    <path d="M12,2A10,10 0 0,0 2,12A10,10 0 0,0 12,22A10,10 0 0,0 22,12A10,10 0 0,0 12,2M12,4A8,8 0 0,1 20,12A8,8 0 0,1 12,20A8,8 0 0,1 4,12A8,8 0 0,1 12,4M12,6A6,6 0 0,0 6,12A6,6 0 0,0 12,18A6,6 0 0,0 18,12A6,6 0 0,0 12,6M12,8A4,4 0 0,1 16,12A4,4 0 0,1 12,16A4,4 0 0,1 8,12A4,4 0 0,1 12,8Z"/>
                </svg>
                Protein Information
            </h3>
            <div class="info-grid">
                <div class="info-item">
                    <span class="info-label">PDB ID</span>
                    <span class="info-value">{pdb_id or 'Custom'}</span>
                </div>
                <div class="info-item">
                    <span class="info-label">Title</span>
                    <span class="info-value">{title[:30]}{'...' if len(title) > 30 else ''}</span>
                </div>
                <div class="info-item">
                    <span class="info-label">Organism</span>
                    <span class="info-value">{organism[:20]}{'...' if len(organism) > 20 else ''}</span>
                </div>
                <div class="info-item">
                    <span class="info-label">Method</span>
                    <span class="info-value">{method}</span>
                </div>
                <div class="info-item">
                    <span class="info-label">Resolution</span>
                    <span class="info-value">{resolution}</span>
                </div>
            </div>
        </div>

        <div class="card">
            <h3 class="card-title">
                <svg class="card-icon" viewBox="0 0 24 24">
                    <path d="M12,2A10,10 0 0,0 2,12A10,10 0 0,0 12,22A10,10 0 0,0 22,12A10,10 0 0,0 12,2M12,4A8,8 0 0,1 20,12A8,8 0 0,1 12,20A8,8 0 0,1 4,12A8,8 0 0,1 12,4M12,6A6,6 0 0,0 6,12A6,6 0 0,0 12,18A6,6 0 0,0 18,12A6,6 0 0,0 12,6M12,8A4,4 0 0,1 16,12A4,4 0 0,1 12,16A4,4 0 0,1 8,12A4,4 0 0,1 12,8Z"/>
                </svg>
                Structure Statistics
            </h3>
            <div class="info-grid">
                <div class="info-item">
                    <span class="info-label">Residues</span>
                    <span class="info-value">{residue_count}</span>
                </div>
                <div class="info-item">
                    <span class="info-label">Atoms</span>
                    <span class="info-value">{atom_count}</span>
                </div>
                <div class="info-item">
                    <span class="info-label">Chains</span>
                    <span class="info-value">1</span>
                </div>
            </div>
        </div>
        """



import json
import os
import sys
from pathlib import Path
from tempfile import TemporaryDirectory

import pytest

os.environ.setdefault("TOOLUNIVERSE_LIGHT_IMPORT", "1")

PROJECT_ROOT = Path(__file__).resolve().parents[1]
SRC_PATH = PROJECT_ROOT / "src"
if str(SRC_PATH) not in sys.path:
    sys.path.insert(0, str(SRC_PATH))

from tooluniverse import ToolUniverse
from tooluniverse.base_tool import BaseTool


class CountingTool(BaseTool):
    call_count = 0
    STATIC_CACHE_VERSION = "1"

    def run(self, arguments, **kwargs):
        CountingTool.call_count += 1
        value = arguments.get("value", 0)
        return {"value": value, "calls": CountingTool.call_count}


@pytest.fixture
def tool_config():
    return {
        "name": "CountingToolTest",
        "type": "CountingTool",
        "description": "Simple counting tool for cache tests",
        "parameter": {
            "type": "object",
            "properties": {"value": {"type": "integer"}},
            "required": ["value"],
        },
    }


def _register_tool(engine: ToolUniverse, tool_config):
    engine.register_custom_tool(CountingTool, tool_config=tool_config)


def _call(engine: ToolUniverse, value: int):
    return engine.run_one_function(
        {"name": "CountingToolTest", "arguments": {"value": value}},
        use_cache=True,
    )

def _with_env(**overrides):
    env_vars = {
        "TOOLUNIVERSE_CACHE_ENABLED": "true",
        "TOOLUNIVERSE_CACHE_PERSIST": "true",
        "TOOLUNIVERSE_CACHE_MEMORY_SIZE": "4",
    }
    env_vars.update(overrides)
    old_env = {k: os.environ.get(k) for k in env_vars}
    os.environ.update(env_vars)
    return env_vars, old_env

def _restore_env(old_env):
    for key, value in old_env.items():
        if value is None:
            os.environ.pop(key, None)
        else:
            os.environ[key] = value


def test_tooluniverse_cache_roundtrip(tool_config):
    with TemporaryDirectory() as tmpdir:
        env_vars, old_env = _with_env(
            TOOLUNIVERSE_CACHE_PATH=str(Path(tmpdir) / "cache.sqlite")
        )
        try:
            CountingTool.call_count = 0
            tu1 = ToolUniverse(tool_files={}, keep_default_tools=False)
            _register_tool(tu1, tool_config)

            result1 = _call(tu1, 7)
            assert result1["value"] == 7
            assert CountingTool.call_count == 1

            result2 = _call(tu1, 7)
            assert result2 == result1
            assert CountingTool.call_count == 1  # cache hit

            tu1.close()

            CountingTool.call_count = 0
            tu2 = ToolUniverse(tool_files={}, keep_default_tools=False)
            _register_tool(tu2, tool_config)

            result3 = _call(tu2, 7)
            assert result3 == result1  # loaded from persistent cache
            assert CountingTool.call_count == 0  # run() not invoked

            tu2.close()
        finally:
            _restore_env(old_env)


def test_cache_invalidation_on_version_change(tool_config):
    with TemporaryDirectory() as tmpdir:
        env_vars, old_env = _with_env(
            TOOLUNIVERSE_CACHE_PATH=str(Path(tmpdir) / "cache.sqlite")
        )
        try:
            CountingTool.call_count = 0
            tu = ToolUniverse(tool_files={}, keep_default_tools=False)
            _register_tool(tu, tool_config)

            first = _call(tu, 1)
            assert CountingTool.call_count == 1

            second = _call(tu, 1)
            assert second == first
            assert CountingTool.call_count == 1

            CountingTool.STATIC_CACHE_VERSION = "2"
            CountingTool.call_count = 0

            third = _call(tu, 1)
            assert third["calls"] == 1  # rerun due to version bump

            tu.close()
        finally:
            CountingTool.STATIC_CACHE_VERSION = "1"
            _restore_env(old_env)


def test_cache_stats_and_clear(tool_config):
    with TemporaryDirectory() as tmpdir:
        env_vars, old_env = _with_env(
            TOOLUNIVERSE_CACHE_PATH=str(Path(tmpdir) / "cache.sqlite")
        )
        try:
            CountingTool.call_count = 0
            tu = ToolUniverse(tool_files={}, keep_default_tools=False)
            _register_tool(tu, tool_config)

            _call(tu, 5)
            stats_before = tu.get_cache_stats()
            assert stats_before["enabled"]

            tu.clear_cache()
            stats_after = tu.get_cache_stats()
            assert stats_after["memory"]["current_size"] == 0

            tu.close()
        finally:
            _restore_env(old_env)


def test_cache_ttl(tool_config):
    with TemporaryDirectory() as tmpdir:
        env_vars, old_env = _with_env(
            TOOLUNIVERSE_CACHE_PATH=str(Path(tmpdir) / "cache.sqlite"),
            TOOLUNIVERSE_CACHE_DEFAULT_TTL="1",
        )
        try:
            CountingTool.call_count = 0
            tu = ToolUniverse(tool_files={}, keep_default_tools=False)
            _register_tool(tu, tool_config)

            first = _call(tu, 9)
            assert CountingTool.call_count == 1

            result_hit = _call(tu, 9)
            assert result_hit == first
            assert CountingTool.call_count == 1

            import time

            time.sleep(1.1)
            result_after_ttl = _call(tu, 9)
            assert result_after_ttl["calls"] == 2  # rerun due to TTL expiry

            tu.close()
        finally:
            _restore_env(old_env)


def test_dump_cache(tool_config):
    with TemporaryDirectory() as tmpdir:
        env_vars, old_env = _with_env(
            TOOLUNIVERSE_CACHE_PATH=str(Path(tmpdir) / "cache.sqlite")
        )
        try:
            CountingTool.call_count = 0
            tu = ToolUniverse(tool_files={}, keep_default_tools=False)
            _register_tool(tu, tool_config)

            _call(tu, 3)
            entries = list(tu.dump_cache())
            assert entries
            assert any(e["namespace"] == "CountingToolTest" for e in entries)

            tu.close()
        finally:
            _restore_env(old_env)


def test_batch_run_deduplicates_work(tool_config):
    with TemporaryDirectory() as tmpdir:
        env_vars, old_env = _with_env(
            TOOLUNIVERSE_CACHE_PATH=str(Path(tmpdir) / "cache.sqlite")
        )
        try:
            CountingTool.call_count = 0
            tu = ToolUniverse(tool_files={}, keep_default_tools=False)
            _register_tool(tu, tool_config)

            batch_calls = [
                {"name": "CountingToolTest", "arguments": {"value": 1}},
                {"name": "CountingToolTest", "arguments": {"value": 1}},
                {"name": "CountingToolTest", "arguments": {"value": 2}},
                {"name": "CountingToolTest", "arguments": {"value": 2}},
            ]

            messages = tu.run(batch_calls, use_cache=True, max_workers=4)

            assert CountingTool.call_count == 2  # one execution per unique arguments

            tool_payloads = [
                json.loads(msg["content"])["content"]
                for msg in messages[1:]
                if msg["role"] == "tool"
            ]
            assert [payload["value"] for payload in tool_payloads] == [1, 1, 2, 2]

            tu.close()
        finally:
            _restore_env(old_env)



"""
STRING Database REST API Tool

This tool provides access to protein-protein interaction data from the STRING
database. STRING is a database of known and predicted protein-protein
interactions.
"""

import requests
from typing import Dict, Any, List
from .base_tool import BaseTool
from .tool_registry import register_tool

STRING_BASE_URL = "https://string-db.org/api"


@register_tool("STRINGRESTTool")
class STRINGRESTTool(BaseTool):
    """
    STRING Database REST API tool.
    Generic wrapper for STRING API endpoints defined in ppi_tools.json.
    """

    def __init__(self, tool_config):
        super().__init__(tool_config)
        fields = tool_config.get("fields", {})
        parameter = tool_config.get("parameter", {})

        self.endpoint_template: str = fields.get("endpoint", "/tsv/network")
        self.required: List[str] = parameter.get("required", [])
        self.output_format: str = fields.get("return_format", "TSV")

    def _build_url(self, arguments: Dict[str, Any]) -> str | Dict[str, Any]:
        """Build URL for STRING API request."""
        url_path = self.endpoint_template
        return STRING_BASE_URL + url_path

    def _build_params(self, arguments: Dict[str, Any]) -> Dict[str, Any]:
        """Build parameters for STRING API request."""
        params = {}

        # Map protein IDs to STRING format
        if "protein_ids" in arguments:
            protein_ids = arguments["protein_ids"]
            if isinstance(protein_ids, list):
                params["identifiers"] = "\r".join(protein_ids)
            else:
                params["identifiers"] = str(protein_ids)

        # Add other parameters
        if "species" in arguments:
            params["species"] = arguments["species"]
        if "confidence_score" in arguments:
            params["required_score"] = int(arguments["confidence_score"] * 1000)
        if "limit" in arguments:
            params["limit"] = arguments["limit"]
        if "network_type" in arguments:
            params["network_type"] = arguments["network_type"]

        return params

    def _make_request(self, url: str, params: Dict[str, Any]) -> Dict[str, Any]:
        """Perform a GET request and handle common errors."""
        try:
            response = requests.get(url, params=params, timeout=30)
            response.raise_for_status()

            if self.output_format == "TSV":
                return self._parse_tsv_response(response.text)
            else:
                return response.json()

        except requests.exceptions.RequestException as e:
            return {"error": f"Request failed: {str(e)}"}
        except Exception as e:
            return {"error": f"Unexpected error: {str(e)}"}

    def _parse_tsv_response(self, text: str) -> Dict[str, Any]:
        """Parse TSV response from STRING API."""
        lines = text.strip().split("\n")
        if len(lines) < 2:
            return {"data": [], "error": "No data returned"}

        # Parse header
        header = lines[0].split("\t")

        # Parse data rows
        data = []
        for line in lines[1:]:
            if line.strip():
                values = line.split("\t")
                row = {}
                for i, value in enumerate(values):
                    if i < len(header):
                        row[header[i]] = value
                data.append(row)

        return {"data": data, "header": header}

    def run(self, arguments: Dict[str, Any]) -> Dict[str, Any]:
        """Execute the tool with given arguments."""
        # Validate required parameters
        for param in self.required:
            if param not in arguments:
                return {"error": f"Missing required parameter: {param}"}

        url = self._build_url(arguments)
        if isinstance(url, dict) and "error" in url:
            return url

        params = self._build_params(arguments)
        return self._make_request(url, params)



"""
embedding_sync_download

Download an embedding database from HuggingFace Hub to local storage. Allows accessing databases ...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def embedding_sync_download(
    action: str,
    repository: str,
    local_name: str,
    overwrite: bool,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    Download an embedding database from HuggingFace Hub to local storage. Allows accessing databases ...

    Parameters
    ----------
    action : str
        Action to download database from HuggingFace
    repository : str
        HuggingFace repository to download from (format: username/repo-name)
    local_name : str
        Local name for the downloaded database (optional, defaults to repo name)
    overwrite : bool
        Whether to overwrite existing local database with same name
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    return get_shared_client().run_one_function(
        {
            "name": "embedding_sync_download",
            "arguments": {
                "action": action,
                "repository": repository,
                "local_name": local_name,
                "overwrite": overwrite,
            },
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["embedding_sync_download"]



"""
FAERS_count_seriousness_by_drug_event

Count the number of adverse event reports classified as serious or non-serious, filtered by drug ...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def FAERS_count_seriousness_by_drug_event(
    medicinalproduct: str,
    patientsex: str,
    patientagegroup: str,
    occurcountry: str,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    Count the number of adverse event reports classified as serious or non-serious, filtered by drug ...

    Parameters
    ----------
    medicinalproduct : str
        Drug name.
    patientsex : str
        Patient sex, leave it blank if you don't want to apply a filter.
    patientagegroup : str
        Patient age group.
    occurcountry : str
        Country where event occurred.
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    return get_shared_client().run_one_function(
        {
            "name": "FAERS_count_seriousness_by_drug_event",
            "arguments": {
                "medicinalproduct": medicinalproduct,
                "patientsex": patientsex,
                "patientagegroup": patientagegroup,
                "occurcountry": occurcountry,
            },
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["FAERS_count_seriousness_by_drug_event"]



"""
get_em_3d_fitting_and_reconstruction_details

Retrieve EM 3D fitting model details and associated 3D reconstruction info for a given PDB entry.
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def get_em_3d_fitting_and_reconstruction_details(
    pdb_id: str,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    Retrieve EM 3D fitting model details and associated 3D reconstruction info for a given PDB entry.

    Parameters
    ----------
    pdb_id : str
        4-character RCSB PDB ID
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    return get_shared_client().run_one_function(
        {
            "name": "get_em_3d_fitting_and_reconstruction_details",
            "arguments": {"pdb_id": pdb_id},
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["get_em_3d_fitting_and_reconstruction_details"]



import requests
from .base_tool import BaseTool
from .tool_registry import register_tool


@register_tool("MedRxivTool")
class MedRxivTool(BaseTool):
    """
    Search medRxiv preprints using medRxiv's API (same interface as bioRxiv).

    Arguments:
        query (str): Search term
        max_results (int): Max results to return (default 10, max 200)
    """

    def __init__(
        self,
        tool_config,
        base_url="https://api.medrxiv.org/details",
    ):
        super().__init__(tool_config)
        self.base_url = base_url

    def run(self, arguments=None):
        arguments = arguments or {}
        query = arguments.get("query")
        max_results = int(arguments.get("max_results", 10))
        if not query:
            return {"error": "`query` parameter is required."}
        return self._search(query, max_results)

    def _search(self, query, max_results):
        # Use date range search for recent preprints
        # Format: /medrxiv/{start_date}/{end_date}/{cursor}/json
        from datetime import datetime, timedelta

        # Search last 365 days to get more comprehensive results
        end_date = datetime.now()
        start_date = end_date - timedelta(days=365)

        url = (
            f"{self.base_url}/medrxiv/"
            f"{start_date.strftime('%Y-%m-%d')}/"
            f"{end_date.strftime('%Y-%m-%d')}/0/json"
        )

        try:
            resp = requests.get(url, timeout=20)
            resp.raise_for_status()
            data = resp.json()
        except requests.RequestException as e:
            return {
                "error": "Network/API error calling medRxiv",
                "reason": str(e),
            }
        except ValueError:
            return {"error": "Failed to decode medRxiv response as JSON"}

        results = []
        # The API returns a dictionary with a 'collection' key
        collection = data.get("collection", [])
        if not isinstance(collection, list):
            return {"error": "Unexpected API response format"}

        for item in collection:
            title = item.get("title")
            authors = item.get("authors", "")
            if isinstance(authors, str):
                authors = [a.strip() for a in authors.split(";") if a.strip()]
            elif isinstance(authors, list):
                authors = [str(a).strip() for a in authors if str(a).strip()]
            else:
                authors = []

            year = None
            date = item.get("date")
            if date and len(date) >= 4 and date[:4].isdigit():
                year = int(date[:4])

            doi = item.get("doi")
            url = f"https://www.medrxiv.org/content/{doi}" if doi else None

            # Filter by query if provided - search in both title and abstract
            if query:
                title_match = query.lower() in (title or "").lower()
                abstract_match = (
                    query.lower() in (item.get("abstract", "") or "").lower()
                )
                if not (title_match or abstract_match):
                    continue

            results.append(
                {
                    "title": title or "Title not available",
                    "authors": (
                        authors if authors else "Author information not available"
                    ),
                    "year": year,
                    "doi": doi or "DOI not available",
                    "url": url or "URL not available",
                    "abstract": item.get("abstract", "Abstract not available"),
                    "source": "medRxiv",
                    "data_quality": {
                        "has_abstract": bool(
                            item.get("abstract")
                            and item.get("abstract") != "Abstract not available"
                        ),
                        "has_authors": bool(authors),
                        "has_year": bool(year),
                        "has_doi": bool(doi),
                        "has_url": bool(url),
                    },
                }
            )

        return results[:max_results]



"""
DailyMed_get_spl_by_setid

Get complete label corresponding to SPL Set ID, returns content in XML or JSON format.
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def DailyMed_get_spl_by_setid(
    setid: str,
    format: str,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    Get complete label corresponding to SPL Set ID, returns content in XML or JSON format.

    Parameters
    ----------
    setid : str
        SPL Set ID to query.
    format : str
        Return format, only supports 'xml'.
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    return get_shared_client().run_one_function(
        {
            "name": "DailyMed_get_spl_by_setid",
            "arguments": {"setid": setid, "format": format},
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["DailyMed_get_spl_by_setid"]



import requests
import fitz
import easyocr
from io import BytesIO
from docx import Document
from PIL import Image
from .uspto_tool import USPTOOpenDataPortalTool
from .tool_registry import register_tool


@register_tool("USPTOPatentDocumentDownloader")
class USPTOPatentDocumentDownloader(USPTOOpenDataPortalTool):
    """
    Fetch and download the abstract (ABST), claims (CLM), and full application text (APP.TEXT)
    PDFs for a given patent application number, following the one-time redirect flow.
    """

    def __init__(self, tool_config):
        super().__init__(tool_config=tool_config)

    def run(self, arguments):
        def ocr_pdf_bytes(pdf_bytes, dpi=300):
            print("Running OCR on PDF bytes...")
            doc = fitz.open(stream=pdf_bytes, filetype="pdf")
            pages_text = []

            # Initialize EasyOCR reader once
            reader = easyocr.Reader(["en"], gpu=True)

            for page in doc:
                # render the page to a PIL image at high resolution
                pix = page.get_pixmap(dpi=dpi)
                img = Image.frombytes("RGB", [pix.width, pix.height], pix.samples)
                img_bytes = BytesIO()
                img.save(img_bytes, format="PNG")
                img_bytes.seek(0)
                results = reader.readtext(img_bytes.getvalue(), detail=0)
                text = "\n".join(results)
                pages_text.append(text.strip())

            doc.close()
            return "\n\n".join(pages_text)

        metadata = super().run(arguments)
        if isinstance(metadata, dict) and metadata.get("error"):
            return metadata

        desired = self.tool_config.get("document")

        docs = metadata.get("documentBag", [])
        if not docs:
            return {"error": "No documents found."}

        result = None
        all_doc_codes = set()
        for doc in docs:
            code = doc.get("documentCode")
            all_doc_codes.add(code)
            if code != desired:
                continue

            plain_text = ""
            pdf_opt = None
            word_opt = None
            for opt in doc.get("downloadOptionBag", []):
                m = opt.get("mimeTypeIdentifier", "").upper()
                if m == "PDF" and not pdf_opt:
                    pdf_opt = opt
                elif m == "MS_WORD" and not word_opt:
                    word_opt = opt

            if word_opt:
                url = word_opt["downloadUrl"]
                resp = requests.get(url, headers=self.headers, timeout=30)
                resp.raise_for_status()
                buf = BytesIO(resp.content)
                docx = Document(buf)
                plain_text = "\n\n".join(
                    p.text for p in docx.paragraphs if p.text.strip()
                )

            if not plain_text and pdf_opt:
                url = pdf_opt["downloadUrl"]
                resp = requests.get(url, headers=self.headers, timeout=30)
                resp.raise_for_status()
                pdf_bytes = resp.content
                pdf_doc = fitz.open(stream=pdf_bytes, filetype="pdf")
                for page in pdf_doc:
                    plain_text += page.get_text().strip()
                pdf_doc.close()

                if plain_text == "":
                    # If no text was extracted, try to extract text from images
                    plain_text = ocr_pdf_bytes(pdf_bytes)

            if plain_text:
                # if plain text is longer than current result, it is probably a better text extraction
                if result is None or len(plain_text) > len(result):
                    result = plain_text

        if result is None:
            return {
                "error": f"Could not parse document with code {desired}. The documents available for this patent are: {', '.join(all_doc_codes)}."
            }
        else:
            # Return the plain text extracted from the PDF
            return {"result": result}


# if __name__ == "__main__":
#     # Example usage
#     tool_config = {
#         "name": "uspto_patent_document_downloader",
#         "description": "Download patent documents and extract text.",
#         "document": "ABST",  # Specify the document type to download
#     }
#     downloader = USPTOPatentDocumentDownloader(tool_config)
#     arguments = {"applicationNumberText": "19053071"}  # Example application number
#     result = downloader.run(arguments)
#     print(result)



"""
Test AgenticTool environment variable integration with Space LLM configuration.

This test module verifies that:
1. Space LLM configuration correctly passes to AgenticTool via environment variables
2. Original environment variables still work as expected
3. Configuration priority is correct for both "default" and "fallback" modes
"""

import os
import pytest
import tempfile
from unittest.mock import patch, MagicMock
from pathlib import Path

from tooluniverse.agentic_tool import AgenticTool


class TestAgenticToolEnvironmentVariables:
    """Test AgenticTool environment variable handling."""

    def setup_method(self):
        """Set up test environment by clearing relevant environment variables."""
        # Clear Space environment variables
        env_vars_to_clear = [
            "TOOLUNIVERSE_LLM_DEFAULT_PROVIDER",
            "TOOLUNIVERSE_LLM_MODEL_DEFAULT",
            "TOOLUNIVERSE_LLM_TEMPERATURE",
            "TOOLUNIVERSE_LLM_MAX_TOKENS",
            "TOOLUNIVERSE_LLM_CONFIG_MODE",
            "GEMINI_MODEL_ID",
            "AGENTIC_TOOL_FALLBACK_CHAIN",
            "VLLM_SERVER_URL",
        ]
        for var in env_vars_to_clear:
            if var in os.environ:
                del os.environ[var]

    def teardown_method(self):
        """Clean up test environment."""
        self.setup_method()

    def test_toolspace_llm_default_provider_env_var(self):
        """Test that TOOLUNIVERSE_LLM_DEFAULT_PROVIDER is correctly read."""
        # Set environment variable
        os.environ["TOOLUNIVERSE_LLM_DEFAULT_PROVIDER"] = "CHATGPT"
        
        # Create AgenticTool with minimal config
        tool_config = {
            "name": "test_tool",
            "prompt": "Test prompt: {input}",
            "input_arguments": ["input"],
            "parameter": {
                "type": "object",
                "properties": {"input": {"type": "string"}},
                "required": ["input"]
            }
        }
        
        with patch.object(AgenticTool, '_try_initialize_api'):
            tool = AgenticTool(tool_config)
            
            # Verify the provider was correctly read
            assert tool._api_type == "CHATGPT"

    def test_toolspace_llm_model_default_env_var(self):
        """Test that TOOLUNIVERSE_LLM_MODEL_DEFAULT is correctly read."""
        # Set environment variable
        os.environ["TOOLUNIVERSE_LLM_MODEL_DEFAULT"] = "gpt-4o"
        
        tool_config = {
            "name": "test_tool",
            "prompt": "Test prompt: {input}",
            "input_arguments": ["input"],
            "parameter": {
                "type": "object",
                "properties": {"input": {"type": "string"}},
                "required": ["input"]
            }
        }
        
        with patch.object(AgenticTool, '_try_initialize_api'):
            tool = AgenticTool(tool_config)
            
            # Verify the model was correctly read
            assert tool._env_model_id == "gpt-4o"

    def test_toolspace_llm_temperature_env_var(self):
        """Test that TOOLUNIVERSE_LLM_TEMPERATURE is correctly read."""
        # Set environment variable
        os.environ["TOOLUNIVERSE_LLM_TEMPERATURE"] = "0.8"
        
        tool_config = {
            "name": "test_tool",
            "prompt": "Test prompt: {input}",
            "input_arguments": ["input"],
            "parameter": {
                "type": "object",
                "properties": {"input": {"type": "string"}},
                "required": ["input"]
            }
        }
        
        with patch.object(AgenticTool, '_try_initialize_api'):
            tool = AgenticTool(tool_config)
            
            # Verify the temperature was correctly read
            assert tool._temperature == 0.8

    def test_max_tokens_handled_by_client(self):
        """Test that max_tokens is handled by LLM client automatically."""
        tool_config = {
            "name": "test_tool",
            "prompt": "Test prompt: {input}",
            "input_arguments": ["input"],
            "parameter": {
                "type": "object",
                "properties": {"input": {"type": "string"}},
                "required": ["input"]
            }
        }
        
        with patch.object(AgenticTool, '_try_initialize_api'):
            tool = AgenticTool(tool_config)
            
            # Verify that max_new_tokens is not used in AgenticTool
            # (it's handled by the LLM client automatically)
            assert not hasattr(tool, '_max_new_tokens')

    def test_toolspace_llm_config_mode_default(self):
        """Test 'default' mode configuration priority."""
        # Set environment variables
        os.environ["TOOLUNIVERSE_LLM_CONFIG_MODE"] = "default"
        os.environ["TOOLUNIVERSE_LLM_DEFAULT_PROVIDER"] = "GEMINI"
        os.environ["TOOLUNIVERSE_LLM_TEMPERATURE"] = "0.9"
        
        tool_config = {
            "name": "test_tool",
            "prompt": "Test prompt: {input}",
            "input_arguments": ["input"],
            "parameter": {
                "type": "object",
                "properties": {"input": {"type": "string"}},
                "required": ["input"]
            },
            # Tool config should override env vars
            "api_type": "CHATGPT",
            "temperature": 0.5
        }
        
        with patch.object(AgenticTool, '_try_initialize_api'):
            tool = AgenticTool(tool_config)
            
            # In default mode, tool config should take priority
            assert tool._api_type == "CHATGPT"  # From tool config
            assert tool._temperature == 0.5     # From tool config

    def test_toolspace_llm_config_mode_fallback(self):
        """Test 'fallback' mode configuration priority."""
        # Set environment variables
        os.environ["TOOLUNIVERSE_LLM_CONFIG_MODE"] = "fallback"
        os.environ["TOOLUNIVERSE_LLM_DEFAULT_PROVIDER"] = "GEMINI"
        os.environ["TOOLUNIVERSE_LLM_TEMPERATURE"] = "0.9"
        
        tool_config = {
            "name": "test_tool",
            "prompt": "Test prompt: {input}",
            "input_arguments": ["input"],
            "parameter": {
                "type": "object",
                "properties": {"input": {"type": "string"}},
                "required": ["input"]
            },
            # Tool config should override env vars
            "api_type": "CHATGPT",
            "temperature": 0.5
        }
        
        with patch.object(AgenticTool, '_try_initialize_api'):
            tool = AgenticTool(tool_config)
            
            # In fallback mode, tool config should take priority
            assert tool._api_type == "CHATGPT"  # From tool config
            assert tool._temperature == 0.5     # From tool config

    def test_original_gemini_model_id_env_var(self):
        """Test that original GEMINI_MODEL_ID environment variable still works."""
        # Set original environment variable
        os.environ["GEMINI_MODEL_ID"] = "gemini-1.5-pro"
        
        tool_config = {
            "name": "test_tool",
            "prompt": "Test prompt: {input}",
            "input_arguments": ["input"],
            "parameter": {
                "type": "object",
                "properties": {"input": {"type": "string"}},
                "required": ["input"]
            }
        }
        
        with patch.object(AgenticTool, '_try_initialize_api'):
            tool = AgenticTool(tool_config)
            
            # Verify the original Gemini model ID was correctly read
            assert tool._gemini_model_id == "gemini-1.5-pro"

    def test_original_agentic_tool_fallback_chain_env_var(self):
        """Test that original AGENTIC_TOOL_FALLBACK_CHAIN environment variable still works."""
        # Set original environment variable
        fallback_chain = [
            {"api_type": "CHATGPT", "model_id": "gpt-4o"},
            {"api_type": "GEMINI", "model_id": "gemini-2.0-flash"}
        ]
        os.environ["AGENTIC_TOOL_FALLBACK_CHAIN"] = str(fallback_chain).replace("'", '"')
        
        tool_config = {
            "name": "test_tool",
            "prompt": "Test prompt: {input}",
            "input_arguments": ["input"],
            "parameter": {
                "type": "object",
                "properties": {"input": {"type": "string"}},
                "required": ["input"]
            }
        }
        
        with patch.object(AgenticTool, '_try_initialize_api'):
            tool = AgenticTool(tool_config)
            
            # Verify the fallback chain was correctly read
            assert len(tool._global_fallback_chain) == 2
            assert tool._global_fallback_chain[0]["api_type"] == "CHATGPT"
            assert tool._global_fallback_chain[1]["api_type"] == "GEMINI"

    def test_original_vllm_server_url_env_var(self):
        """Test that original VLLM_SERVER_URL environment variable still works."""
        # Set original environment variable
        os.environ["VLLM_SERVER_URL"] = "http://localhost:8000"
        
        tool_config = {
            "name": "test_tool",
            "prompt": "Test prompt: {input}",
            "input_arguments": ["input"],
            "parameter": {
                "type": "object",
                "properties": {"input": {"type": "string"}},
                "required": ["input"]
            },
            "api_type": "VLLM"
        }
        
        with patch.object(AgenticTool, '_try_initialize_api'):
            tool = AgenticTool(tool_config)
            
            # Verify the VLLM server URL was correctly read
            # This is tested indirectly through the VLLM client initialization
            assert os.getenv("VLLM_SERVER_URL") == "http://localhost:8000"

    def test_task_specific_model_env_var(self):
        """Test that task-specific model environment variables work."""
        # Set task-specific environment variable
        os.environ["TOOLUNIVERSE_LLM_MODEL_ANALYSIS"] = "gpt-4o"
        os.environ["TOOLUNIVERSE_LLM_MODEL_DEFAULT"] = "gpt-3.5-turbo"
        
        tool_config = {
            "name": "test_tool",
            "prompt": "Test prompt: {input}",
            "input_arguments": ["input"],
            "parameter": {
                "type": "object",
                "properties": {"input": {"type": "string"}},
                "required": ["input"]
            },
            "llm_task": "analysis"  # This should use the analysis-specific model
        }
        
        with patch.object(AgenticTool, '_try_initialize_api'):
            tool = AgenticTool(tool_config)
            
            # Verify the task-specific model was correctly read
            assert tool._env_model_id == "gpt-4o"

    def test_environment_variable_priority_in_default_mode(self):
        """Test that environment variables take priority over defaults in 'default' mode."""
        # Set environment variables
        os.environ["TOOLUNIVERSE_LLM_CONFIG_MODE"] = "default"
        os.environ["TOOLUNIVERSE_LLM_DEFAULT_PROVIDER"] = "GEMINI"
        os.environ["TOOLUNIVERSE_LLM_TEMPERATURE"] = "0.8"
        
        tool_config = {
            "name": "test_tool",
            "prompt": "Test prompt: {input}",
            "input_arguments": ["input"],
            "parameter": {
                "type": "object",
                "properties": {"input": {"type": "string"}},
                "required": ["input"]
            }
            # No tool-level config, should use env vars
        }
        
        with patch.object(AgenticTool, '_try_initialize_api'):
            tool = AgenticTool(tool_config)
            
            # In default mode with no tool config, env vars should be used
            assert tool._api_type == "GEMINI"  # From env var
            assert tool._temperature == 0.8    # From env var

    def test_environment_variable_fallback_in_fallback_mode(self):
        """Test that environment variables are used as fallback in 'fallback' mode."""
        # Set environment variables
        os.environ["TOOLUNIVERSE_LLM_CONFIG_MODE"] = "fallback"
        os.environ["TOOLUNIVERSE_LLM_DEFAULT_PROVIDER"] = "GEMINI"
        os.environ["TOOLUNIVERSE_LLM_TEMPERATURE"] = "0.8"
        
        tool_config = {
            "name": "test_tool",
            "prompt": "Test prompt: {input}",
            "input_arguments": ["input"],
            "parameter": {
                "type": "object",
                "properties": {"input": {"type": "string"}},
                "required": ["input"]
            }
            # No tool-level config, should use built-in defaults
        }
        
        with patch.object(AgenticTool, '_try_initialize_api'):
            tool = AgenticTool(tool_config)
            
            # In fallback mode with no tool config, should use built-in defaults
            assert tool._api_type == "CHATGPT"  # Built-in default
            assert tool._temperature == 0.1     # Built-in default


if __name__ == "__main__":
    pytest.main([__file__])
